@@ -49,6 +49,13 @@ class Mock(object):
         else:
             data = Mock(mapping=self.__mapping)
         return data
+
+    def __iter__(self):
+        return self
+
+    def next(self):
+        raise StopIteration
+
 # pylint: enable=R0903
 
 MOCK_MODULES = [
@@ -232,7 +239,7 @@ on_saltstack = 'SALT_ON_SALTSTACK' in os.environ
 project = 'Salt'
 
 version = salt.version.__version__
-latest_release = '2016.11.0'  # latest release
+latest_release = '2016.11.1'  # latest release
 previous_release = '2016.3.4'  # latest release from previous branch
 previous_release_dir = '2016.3'  # path on web server for previous branch
 next_release = ''  # next release
