@@ -378,7 +378,7 @@ def _netstat_route_linux():
 
 def _netstat_route_freebsd():
     '''
-    Return netstat routing information for FreeBSD and OS X
+    Return netstat routing information for FreeBSD and macOS
     '''
     ret = []
     cmd = 'netstat -f inet -rn | tail -n+5'
@@ -490,7 +490,7 @@ def _netstat_route_sunos():
             'gateway': comps[1],
             'netmask': '',
             'flags': comps[2],
-            'interface': comps[5]})
+            'interface': comps[5] if len(comps) >= 6 else ''})
     cmd = 'netstat -f inet6 -rn | tail -n+5'
     out = __salt__['cmd.run'](cmd, python_shell=True)
     for line in out.splitlines():
@@ -501,7 +501,7 @@ def _netstat_route_sunos():
             'gateway': comps[1],
             'netmask': '',
             'flags': comps[2],
-            'interface': comps[5]})
+            'interface': comps[5] if len(comps) >= 6 else ''})
     return ret
 
 
@@ -993,7 +993,8 @@ def mod_hostname(hostname):
             if 'Static hostname' in line[0]:
                 o_hostname = line[1].strip()
     elif not salt.utils.is_sunos():
-        o_hostname = __salt__['cmd.run']('{0} -f'.format(hostname_cmd))
+        # don't run hostname -f because -f is not supported on all platforms
+        o_hostname = socket.getfqdn()
     else:
         # output: Hostname core OK: fully qualified as core.acheron.be
         o_hostname = __salt__['cmd.run'](check_hostname_cmd).split(' ')[-1]
@@ -1036,7 +1037,7 @@ def mod_hostname(hostname):
                     fh_.write('HOSTNAME={0}\n'.format(hostname))
                 else:
                     fh_.write(net)
-    elif __grains__['os_family'] == 'Debian':
+    elif __grains__['os_family'] in ('Debian', 'NILinuxRT'):
         with salt.utils.fopen('/etc/hostname', 'w') as fh_:
             fh_.write(hostname + '\n')
     elif __grains__['os_family'] == 'OpenBSD':
@@ -1466,3 +1467,49 @@ def get_route(ip):
 
     else:
         raise CommandExecutionError('Not yet supported on this platform')
+
+
+def ifacestartswith(cidr):
+    '''
+    Retrieve the interface name from a specific CIDR
+
+    .. versionadded:: 2016.11.0
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' network.ifacestartswith 10.0
+    '''
+    net_list = interfaces()
+    intfnames = []
+    pattern = str(cidr)
+    size = len(pattern)
+    for ifname, ifval in six.iteritems(net_list):
+        if 'inet' in ifval:
+            for inet in ifval['inet']:
+                if inet['address'][0:size] == pattern:
+                    if 'label' in inet:
+                        intfnames.append(inet['label'])
+                    else:
+                        intfnames.append(ifname)
+    return intfnames
+
+
+def iphexval(ip):
+    '''
+    Retrieve the interface name from a specific CIDR
+
+    .. versionadded:: 2016.11.0
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' network.iphexval 10.0.0.1
+    '''
+    a = ip.split('.')
+    hexval = ""
+    for val in a:
+        hexval = hexval.join(hex(int(val))[2:])
+    return hexval.upper()
