@@ -2,6 +2,12 @@
 '''
 Support for ``pkgng``, the new package manager for FreeBSD
 
+.. important::
+    If you feel that Salt should be using this module to manage packages on a
+    minion, and it is using a different module (or gives an error similar to
+    *'pkg.install' is not available*), see :ref:`here
+    <module-provider-override>`.
+
 .. warning::
 
     This module has been completely rewritten. Up to and including version
@@ -59,8 +65,7 @@ def __virtual__():
         return __virtualname__
     if __grains__['os'] == 'FreeBSD' and float(__grains__['osrelease']) >= 10:
         return __virtualname__
-    if __grains__['os'] == 'FreeBSD' and \
-            float(__grains__['osmajorrelease']) == 9:
+    if __grains__['os'] == 'FreeBSD' and int(__grains__['osmajorrelease']) == 9:
         providers = {}
         if 'providers' in __opts__:
             providers = __opts__['providers']
@@ -69,27 +74,32 @@ def __virtual__():
             log.debug('Configuration option \'providers:pkg\' is set to '
                 '\'pkgng\', using \'pkgng\' in favor of \'freebsdpkg\'.')
             return __virtualname__
-    return False
+    return (False,
+            'The pkgng execution module cannot be loaded: only available '
+            'on FreeBSD 10 or FreeBSD 9 with providers.pkg set to pkgng.')
 
 
-def _pkg(jail=None, chroot=None):
+def _pkg(jail=None, chroot=None, root=None):
     '''
     Returns the prefix for a pkg command, using -j if a jail is specified, or
     -c if chroot is specified.
     '''
-    ret = 'pkg'
+    ret = ['pkg']
     if jail:
-        ret += ' -j {0!r}'.format(jail)
+        ret.extend(['-j', jail])
     elif chroot:
-        ret += ' -c {0!r}'.format(chroot)
+        ret.extend(['-c', chroot])
+    elif root:
+        ret.extend(['-r', root])
     return ret
 
 
-def _get_pkgng_version(jail=None, chroot=None):
+def _get_pkgng_version(jail=None, chroot=None, root=None):
     '''
     return the version of 'pkg'
     '''
-    return __salt__['cmd.run']([_pkg(jail, chroot), '--version']).strip()
+    cmd = _pkg(jail, chroot, root) + ['--version']
+    return __salt__['cmd.run'](cmd).strip()
 
 
 def _get_version(name, results):
@@ -97,7 +107,7 @@ def _get_version(name, results):
     ``pkg search`` will return all packages for which the pattern is a match.
     Narrow this down and return the package version, or None if no exact match.
     '''
-    for line in results.splitlines():
+    for line in salt.utils.itertools.split(results, '\n'):
         if not line:
             continue
         try:
@@ -109,7 +119,7 @@ def _get_version(name, results):
     return None
 
 
-def _contextkey(jail=None, chroot=None, prefix='pkg.list_pkgs'):
+def _contextkey(jail=None, chroot=None, root=None, prefix='pkg.list_pkgs'):
     '''
     As this module is designed to manipulate packages in jails and chroots, use
     the passed jail/chroot to ensure that a key in the __context__ dict that is
@@ -119,6 +129,8 @@ def _contextkey(jail=None, chroot=None, prefix='pkg.list_pkgs'):
         return str(prefix) + '.jail_{0}'.format(jail)
     elif chroot:
         return str(prefix) + '.chroot_{0}'.format(chroot)
+    elif root:
+        return str(prefix) + '.root_{0}'.format(root)
     return prefix
 
 
@@ -167,6 +179,10 @@ def version(*names, **kwargs):
         Get package version information for the specified chroot (ignored if
         ``jail`` is specified)
 
+    root
+        Get package version information for the specified root (ignored if
+        ``jail`` is specified)
+
     with_origin : False
         Return a nested dictionary containing both the origin name and version
         for each specified package.
@@ -199,7 +215,7 @@ def version(*names, **kwargs):
 info = salt.utils.alias_function(version, 'info')
 
 
-def refresh_db(jail=None, chroot=None, force=False):
+def refresh_db(jail=None, chroot=None, root=None, force=False):
     '''
     Refresh PACKAGESITE contents
 
@@ -221,6 +237,10 @@ def refresh_db(jail=None, chroot=None, force=False):
         Refresh the pkg database within the specified chroot (ignored if
         ``jail`` is specified)
 
+    root
+        Refresh the pkg database within the specified root (ignored if
+        ``jail`` is specified)
+
     force
         Force a full download of the repository catalog without regard to the
         respective ages of the local and remote copies of the catalog.
@@ -231,12 +251,11 @@ def refresh_db(jail=None, chroot=None, force=False):
 
             salt '*' pkg.refresh_db force=True
     '''
-    opts = ''
+    cmd = _pkg(jail, chroot, root)
+    cmd.append('update')
     if force:
-        opts += ' -f'
-    return __salt__['cmd.retcode'](
-        '{0} update{1}'.format(_pkg(jail, chroot), opts),
-        python_shell=False) == 0
+        cmd.append('-f')
+    return __salt__['cmd.retcode'](cmd, python_shell=False) == 0
 
 
 # Support pkg.update to refresh the db, since this is the CLI usage
@@ -269,23 +288,36 @@ def latest_version(*names, **kwargs):
         ret[name] = ''
     jail = kwargs.get('jail')
     chroot = kwargs.get('chroot')
-    pkgs = list_pkgs(versions_as_list=True, jail=jail, chroot=chroot)
+    refresh = kwargs.get('refresh')
+    root = kwargs.get('root')
+    pkgs = list_pkgs(versions_as_list=True, jail=jail, chroot=chroot, root=root)
 
-    if salt.utils.compare_versions(_get_pkgng_version(jail, chroot), '>=', '1.6.0'):
+    if salt.utils.compare_versions(_get_pkgng_version(jail, chroot, root), '>=', '1.6.0'):
         quiet = True
     else:
         quiet = False
 
+    cmd_prefix = _pkg(jail, chroot, root) + ['search']
     for name in names:
-        cmd = [_pkg(jail, chroot), 'search']
+        # FreeBSD supports packages in format java/openjdk7
+        if '/' in name:
+            cmd = _pkg(jail, chroot, root) + ['search']
+        else:
+            cmd = _pkg(jail, chroot, root) + ['search', '-S', 'name', '-Q', 'version', '-e']
         if quiet:
             cmd.append('-q')
+        if not salt.utils.is_true(refresh):
+            cmd.append('-U')
         cmd.append(name)
 
         pkgver = _get_version(
             name,
-            __salt__['cmd.run'](cmd, python_shell=False, output_loglevel='trace')
+            sorted(
+                __salt__['cmd.run'](cmd, python_shell=False, output_loglevel='trace').splitlines(),
+                reverse=True
+            ).pop(0)
         )
+
         if pkgver is not None:
             installed = pkgs.get(name, [])
             if not installed:
@@ -312,6 +344,7 @@ available_version = salt.utils.alias_function(latest_version, 'available_version
 def list_pkgs(versions_as_list=False,
               jail=None,
               chroot=None,
+              root=None,
               with_origin=False,
               **kwargs):
     '''
@@ -326,6 +359,10 @@ def list_pkgs(versions_as_list=False,
         List the packages in the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        List the packages in the specified root (ignored if ``jail`` is
+        specified)
+
     with_origin : False
         Return a nested dictionary containing both the origin name and version
         for each installed package.
@@ -346,8 +383,8 @@ def list_pkgs(versions_as_list=False,
         return {}
 
     versions_as_list = salt.utils.is_true(versions_as_list)
-    contextkey_pkg = _contextkey(jail, chroot)
-    contextkey_origins = _contextkey(jail, chroot, prefix='pkg.origin')
+    contextkey_pkg = _contextkey(jail, chroot, root)
+    contextkey_origins = _contextkey(jail, chroot, root, prefix='pkg.origin')
 
     if contextkey_pkg in __context__:
         ret = copy.deepcopy(__context__[contextkey_pkg])
@@ -363,12 +400,11 @@ def list_pkgs(versions_as_list=False,
 
     ret = {}
     origins = {}
-    cmd = '{0} info -ao'.format(_pkg(jail, chroot))
     out = __salt__['cmd.run_stdout'](
-            cmd,
-            python_shell=False,
-            output_loglevel='trace')
-    for line in out.splitlines():
+        _pkg(jail, chroot, root) + ['info', '-ao'],
+        output_loglevel='trace',
+        python_shell=False)
+    for line in salt.utils.itertools.split(out, '\n'):
         if not line:
             continue
         try:
@@ -413,7 +449,7 @@ def update_package_site(new_url):
     return True
 
 
-def stats(local=False, remote=False, jail=None, chroot=None):
+def stats(local=False, remote=False, jail=None, chroot=None, root=None):
     '''
     Return pkgng stats.
 
@@ -456,6 +492,10 @@ def stats(local=False, remote=False, jail=None, chroot=None):
         Retrieve stats from the specified chroot (ignored if ``jail`` is
         specified).
 
+    root
+        Retrieve stats from the specified root (ignored if ``jail`` is
+        specified).
+
         CLI Example:
 
         .. code-block:: bash
@@ -470,19 +510,16 @@ def stats(local=False, remote=False, jail=None, chroot=None):
         opts += 'l'
     if remote:
         opts += 'r'
+
+    cmd = _pkg(jail, chroot, root)
+    cmd.append('stats')
     if opts:
-        opts = '-' + opts
-
-    res = __salt__['cmd.run'](
-        '{0} stats {1}'.format(_pkg(jail, chroot), opts),
-        python_shell=False,
-        output_loglevel='trace'
-    )
-    res = [x.strip("\t") for x in res.split("\n")]
-    return res
+        cmd.append('-' + opts)
+    out = __salt__['cmd.run'](cmd, output_loglevel='trace', python_shell=False)
+    return [x.strip('\t') for x in salt.utils.itertools.split(out, '\n')]
 
 
-def backup(file_name, jail=None, chroot=None):
+def backup(file_name, jail=None, chroot=None, root=None):
     '''
     Export installed packages into yaml+mtree file
 
@@ -509,21 +546,27 @@ def backup(file_name, jail=None, chroot=None):
         so the path to the backup file will be relative to the root of the
         chroot.
 
+    root
+        Backup packages from the specified root (ignored if ``jail`` is
+        specified). Note that this will run the command within the root, and
+        so the path to the backup file will be relative to the root of the
+        root.
+
         CLI Example:
 
         .. code-block:: bash
 
             salt '*' pkg.backup /tmp/pkg chroot=/path/to/chroot
     '''
-    res = __salt__['cmd.run'](
-        '{0} backup -d {1!r}'.format(_pkg(jail, chroot), file_name),
-        python_shell=False,
-        output_loglevel='trace'
+    ret = __salt__['cmd.run'](
+        _pkg(jail, chroot, root) + ['backup', '-d', file_name],
+        output_loglevel='trace',
+        python_shell=False
     )
-    return res.split('...')[1]
+    return ret.split('...')[1]
 
 
-def restore(file_name, jail=None, chroot=None):
+def restore(file_name, jail=None, chroot=None, root=None):
     '''
     Reads archive created by pkg backup -d and recreates the database.
 
@@ -550,6 +593,12 @@ def restore(file_name, jail=None, chroot=None):
         so the path to the file from which the pkg database will be restored is
         relative to the root of the chroot.
 
+    root
+        Restore database to the specified root (ignored if ``jail`` is
+        specified). Note that this will run the command within the root, and
+        so the path to the file from which the pkg database will be restored is
+        relative to the root of the root.
+
         CLI Example:
 
         .. code-block:: bash
@@ -557,13 +606,13 @@ def restore(file_name, jail=None, chroot=None):
             salt '*' pkg.restore /tmp/pkg chroot=/path/to/chroot
     '''
     return __salt__['cmd.run'](
-        '{0} backup -r {1!r}'.format(_pkg(jail, chroot), file_name),
-        python_shell=False,
-        output_loglevel='trace'
+        _pkg(jail, chroot, root) + ['backup', '-r', file_name],
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
-def audit(jail=None, chroot=None):
+def audit(jail=None, chroot=None, root=None):
     '''
     Audits installed packages against known vulnerabilities
 
@@ -586,6 +635,10 @@ def audit(jail=None, chroot=None):
         Audit packages within the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Audit packages within the specified root (ignored if ``jail`` is
+        specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -593,9 +646,9 @@ def audit(jail=None, chroot=None):
             salt '*' pkg.audit chroot=/path/to/chroot
     '''
     return __salt__['cmd.run'](
-        '{0} audit -F'.format(_pkg(jail, chroot)),
-        python_shell=False,
-        output_loglevel='trace'
+        _pkg(jail, chroot, root) + ['audit', '-F'],
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
@@ -605,6 +658,7 @@ def install(name=None,
             sources=None,
             jail=None,
             chroot=None,
+            root=None,
             orphan=False,
             force=False,
             glob=False,
@@ -634,6 +688,10 @@ def install(name=None,
         Install the package into the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Install the package into the specified root (ignored if ``jail`` is
+        specified)
+
     orphan
         Mark the installed package as orphan. Will be automatically removed
         if no other packages depend on them. For more information please
@@ -746,7 +804,6 @@ def install(name=None,
         return {}
 
     opts = 'y'
-    repo_opts = ''
     if salt.utils.is_true(orphan):
         opts += 'A'
     if salt.utils.is_true(force):
@@ -761,18 +818,12 @@ def install(name=None,
         opts += 'q'
     if salt.utils.is_true(reinstall_requires):
         opts += 'R'
-    if fromrepo:
-        repo_opts += 'r {0}'.format(fromrepo)
     if salt.utils.is_true(regex):
         opts += 'x'
     if salt.utils.is_true(pcre):
         opts += 'X'
-    if opts:
-        opts = '-' + opts
-    if repo_opts:
-        repo_opts = '-' + repo_opts
 
-    old = list_pkgs(jail=jail, chroot=chroot)
+    old = list_pkgs(jail=jail, chroot=chroot, root=root)
 
     if pkg_type == 'file':
         pkg_cmd = 'add'
@@ -792,25 +843,48 @@ def install(name=None,
             else:
                 targets.append('{0}-{1}'.format(param, version_num))
 
-    cmd = '{0} {1} {2} {3} {4}'.format(
-        _pkg(jail, chroot), pkg_cmd, repo_opts, opts, ' '.join(targets)
-    )
+    cmd = _pkg(jail, chroot, root)
+    cmd.append(pkg_cmd)
+    if fromrepo:
+        cmd.extend(['-r', fromrepo])
+    if opts:
+        cmd.append('-' + opts)
+    cmd.extend(targets)
 
     if pkg_cmd == 'add' and salt.utils.is_true(dryrun):
         # pkg add doesn't have a dryrun mode, so echo out what will be run
-        return cmd
+        return ' '.join(cmd)
 
-    __salt__['cmd.run'](cmd, python_shell=False, output_loglevel='trace')
-    __context__.pop(_contextkey(jail, chroot), None)
-    __context__.pop(_contextkey(jail, chroot, prefix='pkg.origin'), None)
-    new = list_pkgs(jail=jail, chroot=chroot)
-    return salt.utils.compare_dicts(old, new)
+    out = __salt__['cmd.run_all'](
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
+    )
+
+    if out['retcode'] != 0 and out['stderr']:
+        errors = [out['stderr']]
+    else:
+        errors = []
+
+    __context__.pop(_contextkey(jail, chroot, root), None)
+    __context__.pop(_contextkey(jail, chroot, root, prefix='pkg.origin'), None)
+    new = list_pkgs(jail=jail, chroot=chroot, root=root)
+    ret = salt.utils.compare_dicts(old, new)
+
+    if errors:
+        raise CommandExecutionError(
+            'Problem encountered installing package(s)',
+            info={'errors': errors, 'changes': ret}
+        )
+
+    return ret
 
 
 def remove(name=None,
            pkgs=None,
            jail=None,
            chroot=None,
+           root=None,
            all_installed=False,
            force=False,
            glob=False,
@@ -843,6 +917,10 @@ def remove(name=None,
         Delete the package from the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Delete the package from the specified root (ignored if ``jail`` is
+        specified)
+
     all_installed
         Deletes all installed packages from the system and empties the
         database. USE WITH CAUTION!
@@ -915,12 +993,12 @@ def remove(name=None,
         raise CommandExecutionError(exc)
 
     targets = []
-    old = list_pkgs(jail=jail, chroot=chroot, with_origin=True)
+    old = list_pkgs(jail=jail, chroot=chroot, root=root, with_origin=True)
     for pkg in pkg_params.items():
         # FreeBSD pkg supports `openjdk` and `java/openjdk7` package names
         if pkg[0].find("/") > 0:
             origin = pkg[0]
-            pkg = [k for k, v in old.iteritems() if v['origin'] == origin][0]
+            pkg = [k for k, v in six.iteritems(old) if v['origin'] == origin][0]
 
         if pkg[0] in old:
             targets.append(pkg[0])
@@ -945,17 +1023,36 @@ def remove(name=None,
         opts += 'x'
     if salt.utils.is_true(pcre):
         opts += 'X'
-    if opts:
-        opts = '-' + opts
 
-    cmd = '{0} delete {1} {2}'.format(
-        _pkg(jail, chroot), opts, ' '.join(targets)
+    cmd = _pkg(jail, chroot, root)
+    cmd.append('delete')
+    if opts:
+        cmd.append('-' + opts)
+    cmd.extend(targets)
+
+    out = __salt__['cmd.run_all'](
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
-    __salt__['cmd.run'](cmd, python_shell=False, output_loglevel='trace')
-    __context__.pop(_contextkey(jail, chroot), None)
-    __context__.pop(_contextkey(jail, chroot, prefix='pkg.origin'), None)
-    new = list_pkgs(jail=jail, chroot=chroot, with_origin=True)
-    return salt.utils.compare_dicts(old, new)
+
+    if out['retcode'] != 0 and out['stderr']:
+        errors = [out['stderr']]
+    else:
+        errors = []
+
+    __context__.pop(_contextkey(jail, chroot, root), None)
+    __context__.pop(_contextkey(jail, chroot, root, prefix='pkg.origin'), None)
+    new = list_pkgs(jail=jail, chroot=chroot, root=root, with_origin=True)
+    ret = salt.utils.compare_dicts(old, new)
+
+    if errors:
+        raise CommandExecutionError(
+            'Problem encountered removing package(s)',
+            info={'errors': errors, 'changes': ret}
+        )
+
+    return ret
 
 # Support pkg.delete to remove packages, since this is the CLI usage
 delete = salt.utils.alias_function(remove, 'delete')
@@ -968,6 +1065,14 @@ def upgrade(*names, **kwargs):
     Upgrade named or all packages (run a ``pkg upgrade``). If <package name> is
     omitted, the operation is executed on all packages.
 
+    Returns a dictionary containing the changes:
+
+    .. code-block:: python
+
+        {'<package>':  {'old': '<old-version>',
+                        'new': '<new-version>'}}
+
+
     CLI Example:
 
     .. code-block:: bash
@@ -987,6 +1092,10 @@ def upgrade(*names, **kwargs):
         Audit packages within the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Audit packages within the specified root (ignored if ``jail`` is
+        specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -1027,13 +1136,9 @@ def upgrade(*names, **kwargs):
 
             salt '*' pkg.upgrade <package name> dryrun=True
     '''
-    ret = {'changes': {},
-           'result': True,
-           'comment': '',
-           }
-
     jail = kwargs.pop('jail', None)
     chroot = kwargs.pop('chroot', None)
+    root = kwargs.pop('root', None)
     force = kwargs.pop('force', False)
     local = kwargs.pop('local', False)
     dryrun = kwargs.pop('dryrun', False)
@@ -1049,26 +1154,31 @@ def upgrade(*names, **kwargs):
     if opts:
         opts = '-' + opts
 
+    cmd = _pkg(jail, chroot, root)
+    cmd.append('upgrade')
+    if opts:
+        cmd.append('-' + opts)
+    cmd.extend(names)
+
     old = list_pkgs()
-    call = __salt__['cmd.run_all'](
-        '{0} upgrade {1} {2}'.format(_pkg(jail, chroot), opts, ' '.join(names)),
-        python_shell=False,
-        output_loglevel='trace'
-    )
-    if call['retcode'] != 0:
-        ret['result'] = False
-        if 'stderr' in call:
-            ret['comment'] += call['stderr']
-        if 'stdout' in call:
-            ret['comment'] += call['stdout']
-    else:
-        __context__.pop('pkg.list_pkgs', None)
-        new = list_pkgs()
-        ret['changes'] = salt.utils.compare_dicts(old, new)
+    result = __salt__['cmd.run_all'](cmd,
+                                     output_loglevel='trace',
+                                     python_shell=False)
+    __context__.pop(_contextkey(jail, chroot, root), None)
+    __context__.pop(_contextkey(jail, chroot, root, prefix='pkg.origin'), None)
+    new = list_pkgs()
+    ret = salt.utils.compare_dicts(old, new)
+
+    if result['retcode'] != 0:
+        raise CommandExecutionError(
+            'Problem encountered upgrading packages',
+            info={'changes': ret, 'result': result}
+        )
+
     return ret
 
 
-def clean(jail=None, chroot=None):
+def clean(jail=None, chroot=None, root=None):
     '''
     Cleans the local cache of fetched remote packages
 
@@ -1081,13 +1191,13 @@ def clean(jail=None, chroot=None):
         salt '*' pkg.clean chroot=/path/to/chroot
     '''
     return __salt__['cmd.run'](
-        '{0} clean'.format(_pkg(jail, chroot)),
-        python_shell=False,
-        output_loglevel='trace'
+        _pkg(jail, chroot, root) + ['clean'],
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
-def autoremove(jail=None, chroot=None, dryrun=False):
+def autoremove(jail=None, chroot=None, root=None, dryrun=False):
     '''
     Delete packages which were automatically installed as dependencies and are
     not required anymore.
@@ -1110,17 +1220,21 @@ def autoremove(jail=None, chroot=None, dryrun=False):
         opts += 'n'
     else:
         opts += 'y'
+
+    cmd = _pkg(jail, chroot, root)
+    cmd.append('autoremove')
     if opts:
-        opts = '-' + opts
+        cmd.append('-' + opts)
     return __salt__['cmd.run'](
-        '{0} autoremove {1}'.format(_pkg(jail, chroot), opts),
-        python_shell=False,
-        output_loglevel='trace'
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
 def check(jail=None,
           chroot=None,
+          root=None,
           depends=False,
           recompute=False,
           checksum=False):
@@ -1140,6 +1254,10 @@ def check(jail=None,
         Perform the sanity check in the specified chroot (ignored if ``jail``
         is specified)
 
+    root
+        Perform the sanity check in the specified root (ignored if ``jail``
+        is specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -1186,17 +1304,19 @@ def check(jail=None,
         opts += 'r'
     if checksum:
         opts += 's'
-    if opts:
-        opts = '-' + opts
 
+    cmd = _pkg(jail, chroot, root)
+    cmd.append('check')
+    if opts:
+        cmd.append('-' + opts)
     return __salt__['cmd.run'](
-        '{0} check {1}'.format(_pkg(jail, chroot), opts),
-        python_shell=False,
-        output_loglevel='trace'
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
-def which(path, jail=None, chroot=None, origin=False, quiet=False):
+def which(path, jail=None, chroot=None, root=None, origin=False, quiet=False):
     '''
     Displays which package installed a specific file
 
@@ -1219,6 +1339,10 @@ def which(path, jail=None, chroot=None, origin=False, quiet=False):
         Perform the check in the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Perform the check in the specified root (ignored if ``jail`` is
+        specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -1249,18 +1373,23 @@ def which(path, jail=None, chroot=None, origin=False, quiet=False):
         opts += 'q'
     if origin:
         opts += 'o'
+
+    cmd = _pkg(jail, chroot, root)
+    cmd.append('which')
     if opts:
-        opts = '-' + opts
+        cmd.append('-' + opts)
+    cmd.append(path)
     return __salt__['cmd.run'](
-        '{0} which {1} {2}'.format(_pkg(jail, chroot), opts, path),
-        python_shell=False,
-        output_loglevel='trace'
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
 def search(name,
            jail=None,
            chroot=None,
+           root=None,
            exact=False,
            glob=False,
            regex=False,
@@ -1295,6 +1424,10 @@ def search(name,
         Perform the search using the ``pkg.conf(5)`` from the specified chroot
         (ignored if ``jail`` is specified)
 
+    root
+        Perform the search using the ``pkg.conf(5)`` from the specified root
+        (ignored if ``jail`` is specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -1436,19 +1569,23 @@ def search(name,
         opts += 'o'
     if prefix:
         opts += 'p'
-    if opts:
-        opts = '-' + opts
 
+    cmd = _pkg(jail, chroot, root)
+    cmd.append('search')
+    if opts:
+        cmd.append('-' + opts)
+    cmd.append(name)
     return __salt__['cmd.run'](
-        '{0} search {1} {2}'.format(_pkg(jail, chroot), opts, name),
-        python_shell=False,
-        output_loglevel='trace'
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
 def fetch(name,
           jail=None,
           chroot=None,
+          root=None,
           fetch_all=False,
           quiet=False,
           fromrepo=None,
@@ -1479,6 +1616,10 @@ def fetch(name,
         Fetch package in the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Fetch package in the specified root (ignored if ``jail`` is
+        specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -1560,13 +1701,10 @@ def fetch(name,
             salt '*' pkg.fetch <package name> depends=True
     '''
     opts = ''
-    repo_opts = ''
     if fetch_all:
         opts += 'a'
     if quiet:
         opts += 'q'
-    if fromrepo:
-        repo_opts += 'r {0}'.format(fromrepo)
     if glob:
         opts += 'g'
     if regex:
@@ -1577,23 +1715,25 @@ def fetch(name,
         opts += 'L'
     if depends:
         opts += 'd'
-    if opts:
-        opts = '-' + opts
-    if repo_opts:
-        repo_opts = '-' + repo_opts
 
+    cmd = _pkg(jail, chroot, root)
+    cmd.extend(['fetch', '-y'])
+    if fromrepo:
+        cmd.extend(['-r', fromrepo])
+    if opts:
+        cmd.append('-' + opts)
+    cmd.append(name)
     return __salt__['cmd.run'](
-        '{0} fetch -y {1} {2} {3}'.format(
-            _pkg(jail, chroot), opts, repo_opts, name
-        ),
-        python_shell=False,
-        output_loglevel='trace'
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
 def updating(name,
              jail=None,
              chroot=None,
+             root=None,
              filedate=None,
              filename=None):
     ''''
@@ -1618,6 +1758,10 @@ def updating(name,
         Perform the action in the specified chroot (ignored if ``jail`` is
         specified)
 
+    root
+        Perform the action in the specified root (ignored if ``jail`` is
+        specified)
+
         CLI Example:
 
         .. code-block:: bash
@@ -1648,11 +1792,46 @@ def updating(name,
         opts += 'd {0}'.format(filedate)
     if filename:
         opts += 'f {0}'.format(filename)
-    if opts:
-        opts = '-' + opts
 
+    cmd = _pkg(jail, chroot, root)
+    cmd.append('updating')
+    if opts:
+        cmd.append('-' + opts)
+    cmd.append(name)
     return __salt__['cmd.run'](
-        '{0} updating {1} {2}'.format(_pkg(jail, chroot), opts, name),
-        python_shell=False,
-        output_loglevel='trace'
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
+
+
+def version_cmp(pkg1, pkg2, ignore_epoch=False):
+    '''
+    Do a cmp-style comparison on two packages. Return -1 if pkg1 < pkg2, 0 if
+    pkg1 == pkg2, and 1 if pkg1 > pkg2. Return None if there was a problem
+    making the comparison.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' pkg.version_cmp '2.1.11' '2.1.12'
+    '''
+    # Don't worry about ignore_epoch since we're shelling out to pkg.
+    sym = {
+        '<': -1,
+        '>': 1,
+        '=': 0,
+    }
+    try:
+        cmd = ['pkg', 'version', '--test-version', pkg1, pkg2]
+        ret = __salt__['cmd.run_all'](cmd,
+                                            output_loglevel='trace',
+                                            python_shell=False,
+                                            ignore_retcode=True)
+    except Exception as exc:
+        log.error(exc)
+
+    if ret['stdout'] in sym:
+        return sym[ret['stdout']]
+    return None
