@@ -17,7 +17,7 @@ A REST API for Salt
       CherryPy milestone 3.3, but the patch was committed for version 3.6.1.
 :optdepends:    - ws4py Python module for websockets support.
 :client_libraries:
-    - Java: https://github.com/SUSE/saltstack-netapi-client-java
+    - Java: https://github.com/SUSE/salt-netapi-client
     - Python: https://github.com/saltstack/pepper
 :setup:
     All steps below are performed on the machine running the Salt Master
@@ -72,10 +72,23 @@ A REST API for Salt
     debug : ``False``
         Starts the web server in development mode. It will reload itself when
         the underlying code is changed and will output more debugging info.
+    log_access_file
+        Path to a file to write HTTP access logs.
+
+        .. versionaddedd:: 2016.11.0
+
+    log_error_file
+        Path to a file to write HTTP error logs.
+
+        .. versionaddedd:: 2016.11.0
+
     ssl_crt
         The path to a SSL certificate. (See below)
     ssl_key
         The path to the private key for your SSL certificate. (See below)
+    ssl_chain
+        (Optional when using PyOpenSSL) the certificate chain to pass to
+        ``Context.load_verify_locations``.
     disable_ssl
         A flag to disable SSL. Warning: your Salt authentication credentials
         will be sent in the clear!
@@ -125,7 +138,8 @@ Authentication
 Authentication is performed by passing a session token with each request.
 Tokens are generated via the :py:class:`Login` URL.
 
-The token may be sent in one of two ways:
+The token may be sent in one of two ways: as a custom header or as a session
+cookie. The latter is far more convenient for clients that support cookies.
 
 * Include a custom header named :mailheader:`X-Auth-Token`.
 
@@ -173,59 +187,233 @@ The token may be sent in one of two ways:
             -d tgt='*' \\
             -d fun=test.ping
 
+  Another example using the :program:`requests` library in Python:
+
+  .. code-block:: python
+
+      >>> import requests
+      >>> session = requests.Session()
+      >>> session.post('http://localhost:8000/login', json={
+          'username': 'saltdev',
+          'password': 'saltdev',
+          'eauth': 'auto',
+      })
+      <Response [200]>
+      >>> resp = session.post('http://localhost:8000', json=[{
+          'client': 'local',
+          'tgt': '*',
+          'fun': 'test.arg',
+          'arg': ['foo', 'bar'],
+          'kwarg': {'baz': 'Baz!'},
+      }])
+      >>> resp.json()
+      {u'return': [{
+          ...snip...
+      }]}
+
 .. seealso:: You can bypass the session handling via the :py:class:`Run` URL.
 
 Usage
 -----
 
-Commands are sent to a running Salt master via this module by sending HTTP
-requests to the URLs detailed below.
+This interface directly exposes Salt's :ref:`Python API <python-api>`.
+Everything possible at the CLI is possible through the Python API. Commands are
+executed on the Salt Master.
 
-.. admonition:: Content negotiation
+The root URL (``/``) is RPC-like in that it accepts instructions in the request
+body for what Salt functions to execute, and the response contains the result
+of those function calls.
 
-    This REST interface is flexible in what data formats it will accept as well
-    as what formats it will return (e.g., JSON, YAML, x-www-form-urlencoded).
+For example:
 
-    * Specify the format of data in the request body by including the
-      :mailheader:`Content-Type` header.
-    * Specify the desired data format for the response body with the
-      :mailheader:`Accept` header.
+.. code-block:: text
 
-Data sent in :http:method:`post` and :http:method:`put` requests  must be in
-the format of a list of lowstate dictionaries. This allows multiple commands to
-be executed in a single HTTP request. The order of commands in the request
-corresponds to the return for each command in the response.
+    % curl -sSi https://localhost:8000 \
+        -H 'Content-type: application/json' \
+        -d '[{
+            "client": "local",
+            "tgt": "*",
+            "fun": "test.ping"
+        }]'
+    HTTP/1.1 200 OK
+    Content-Type: application/json
+    [...snip...]
 
-Lowstate, broadly, is a dictionary of values that are mapped to a function
-call. This pattern is used pervasively throughout Salt. The functions called
-from netapi modules are described in :ref:`Client Interfaces <netapi-clients>`.
+    {"return": [{"jerry": true}]}
 
-The following example (in JSON format) causes Salt to execute two commands, a
-command sent to minions as well as a runner function on the master::
+The request body must be an array of commands. Use this workflow to build a
+command:
 
-    [{
-        "client": "local",
-        "tgt": "*",
-        "fun": "test.fib",
-        "arg": ["10"]
-    },
+1.  Choose a client interface.
+2.  Choose a function.
+3.  Fill out the remaining parameters needed for the chosen client.
+
+The ``client`` field is a reference to the main Python classes used in Salt's
+Python API. Read the full :ref:`client interfaces <netapi-clients>`
+documentation, but in short:
+
+* "local" uses :py:class:`LocalClient <salt.client.LocalClient>` which sends
+  commands to Minions. Equivalent to the ``salt`` CLI command.
+* "runner" uses :py:class:`RunnerClient <salt.runner.RunnerClient>` which
+  invokes runner modules on the Master. Equivalent to the ``salt-run`` CLI
+  command.
+* "wheel" uses :py:class:`WheelClient <salt.wheel.WheelClient>` which invokes
+  wheel modules on the Master. Wheel modules do not have a direct CLI
+  equivalent but they typically manage Master-side resources such as state
+  files, pillar files, the Salt config files, and the :py:mod:`key wheel module
+  <salt.wheel.key>` exposes similar functionality as the ``salt-key`` CLI
+  command.
+
+Most clients have variants like synchronous or asynchronous execution as well as
+others like batch execution. See the :ref:`full list of client interfaces
+<netapi-clients>`.
+
+Each client requires different arguments and sometimes has different syntax.
+For example, ``LocalClient`` requires the ``tgt`` argument because it forwards
+the command to Minions and the other client interfaces do not. ``LocalClient``
+also takes ``arg`` (array) and ``kwarg`` (dictionary) arguments because these
+values are sent to the Minions and used to execute the requested function
+there. ``RunnerClient`` and ``WheelClient`` are executed directly on the Master
+and thus do not need or accept those arguments.
+
+Read the method signatures in the client documentation linked above, but
+hopefully an example will help illustrate the concept. This example causes Salt
+to execute two functions -- the :py:func:`test.arg execution function
+<salt.modules.test.arg>` using ``LocalClient`` and the :py:func:`test.arg
+runner function <salt.runners.test.arg>` using ``RunnerClient``; note the
+different structure for each command. The results for both are combined and
+returned as one response.
+
+.. code-block:: text
+
+    % curl -b ~/cookies.txt -sSi localhost:8000 \
+        -H 'Content-type: application/json' \
+        -d '
+    [
+        {
+            "client": "local",
+            "tgt": "*",
+            "fun": "test.arg",
+            "arg": ["positional arg one", "positional arg two"],
+            "kwarg": {
+                "keyword arg one": "Hello from a minion",
+                "keyword arg two": "Hello again from a minion"
+            }
+        },
+        {
+            "client": "runner",
+            "fun": "test.arg",
+            "keyword arg one": "Hello from a master",
+            "keyword arg two": "Runners do not support positional args"
+        }
+    ]
+    '
+    HTTP/1.1 200 OK
+    [...snip...]
     {
-        "client": "runner",
-        "fun": "jobs.lookup_jid",
-        "jid": "20130603122505459265"
-    }]
+      "return": [
+        {
+          "jerry": {
+            "args": [
+              "positional arg one",
+              "positional arg two"
+            ],
+            "kwargs": {
+              "keyword arg one": "Hello from a minion",
+              "keyword arg two": "Hello again from a minion",
+              [...snip...]
+            }
+          },
+          [...snip; other minion returns here...]
+        },
+        {
+          "args": [],
+          "kwargs": {
+            "keyword arg two": "Runners do not support positional args",
+            "keyword arg one": "Hello from a master"
+          }
+        }
+      ]
+    }
 
-.. admonition:: x-www-form-urlencoded
+One more example, this time with more commonly used functions:
 
-    Sending JSON or YAML in the request body is simple and most flexible,
-    however sending data in urlencoded format is also supported with the
-    caveats below. It is the default format for HTML forms, many JavaScript
-    libraries, and the :command:`curl` command.
+.. code-block:: text
 
-    For example, the equivalent to running ``salt '*' test.ping`` is sending
-    ``fun=test.ping&arg&client=local&tgt=*`` in the HTTP request body.
+    curl -b /tmp/cookies.txt -sSi localhost:8000 \
+        -H 'Content-type: application/json' \
+        -d '
+    [
+        {
+            "client": "local",
+            "tgt": "*",
+            "fun": "state.sls",
+            "kwarg": {
+                "mods": "apache",
+                "pillar": {
+                    "lookup": {
+                        "wwwdir": "/srv/httpd/htdocs"
+                    }
+                }
+            }
+        },
+        {
+            "client": "runner",
+            "fun": "cloud.create",
+            "provider": "my-ec2-provider",
+            "instances": "my-centos-6",
+            "image": "ami-1624987f",
+            "delvol_on_destroy", true
+        }
+    ]
+    '
+    HTTP/1.1 200 OK
+    [...snip...]
+    {
+      "return": [
+        {
+          "jerry": {
+            "pkg_|-install_apache_|-httpd_|-installed": {
+                [...snip full state return here...]
+            }
+          }
+          [...snip other minion returns here...]
+        },
+        {
+            [...snip full salt-cloud output here...]
+        }
+      ]
+    }
 
-    Caveats:
+Content negotiation
+-------------------
+
+This REST interface is flexible in what data formats it will accept as well
+as what formats it will return (e.g., JSON, YAML, urlencoded).
+
+* Specify the format of data in the request body by including the
+  :mailheader:`Content-Type` header.
+* Specify the desired data format for the response body with the
+  :mailheader:`Accept` header.
+
+We recommend the JSON format for most HTTP requests. urlencoded data is simple
+and cannot express complex data structures -- and that is often required for
+some Salt commands, such as starting a state run that uses Pillar data. Salt's
+CLI tool can reformat strings passed in at the CLI into complex data
+structures, and that behavior also works via salt-api, but that can be brittle
+and since salt-api can accept JSON it is best just to send JSON.
+
+Here is an example of sending urlencoded data:
+
+.. code-block:: bash
+
+    curl -sSik https://localhost:8000 \\
+        -b ~/cookies.txt \\
+        -d client=runner \\
+        -d fun='jobs.lookup_jid' \\
+        -d jid='20150129182456704682'
+
+.. admonition:: urlencoded data caveats
 
     * Only a single command may be sent per HTTP request.
     * Repeating the ``arg`` parameter multiple times will cause those
@@ -233,9 +421,23 @@ command sent to minions as well as a runner function on the master::
 
       Note, some popular frameworks and languages (notably jQuery, PHP, and
       Ruby on Rails) will automatically append empty brackets onto repeated
-      parameters. E.g., ``arg=one``, ``arg=two`` will be sent as ``arg[]=one``,
-      ``arg[]=two``. This is not supported; send JSON or YAML instead.
+      query string parameters. E.g., ``?foo[]=fooone&foo[]=footwo``. This is
+      **not** supported; send ``?foo=fooone&foo=footwo`` instead, or send JSON
+      or YAML.
 
+    A note about ``curl``
+
+    The ``-d`` flag to curl does *not* automatically urlencode data which can
+    affect passwords and other data that contains characters that must be
+    encoded. Use the ``--data-urlencode`` flag instead. E.g.:
+
+    .. code-block:: bash
+
+        curl -ksi http://localhost:8000/login \\
+        -H "Accept: application/json" \\
+        -d username='myapiuser' \\
+        --data-urlencode password='1234+' \\
+        -d eauth='pam'
 
 .. |req_token| replace:: a session token from :py:class:`~Login`.
 .. |req_accept| replace:: the desired response format.
@@ -245,24 +447,10 @@ command sent to minions as well as a runner function on the master::
     :mailheader:`Accept` request header.
 
 .. |200| replace:: success
+.. |400| replace:: bad or malformed request
 .. |401| replace:: authentication required
 .. |406| replace:: requested Content-Type not available
 
-A Note About Curl
-=================
-
-When sending passwords and data that might need to be urlencoded, you must set
-the ``-d`` flag to indicate the content type, and the ``--data-urlencode`` flag
-to urlencode the input.
-
-.. code-block:: bash
-
-    curl -ksi http://localhost:8000/login \\
-    -H "Accept: application/json" \\
-    -d username='myapiuser' \\
-    --data-urlencode password='1234+' \\
-    -d eauth='pam'
-
 '''
 # We need a custom pylintrc here...
 # pylint: disable=W0212,E1101,C0103,R0201,W0221,W0613
@@ -274,7 +462,8 @@ import itertools
 import functools
 import logging
 import json
-import StringIO
+import os
+import signal
 import tarfile
 import time
 from multiprocessing import Process, Pipe
@@ -445,11 +634,7 @@ def salt_ip_verify_tool():
                 logger.debug("Request from IP: {0}".format(rem_ip))
                 if rem_ip not in auth_ip_list:
                     logger.error("Blocked IP: {0}".format(rem_ip))
-                    cherrypy.response.status = 403
-                    return {
-                        'status': cherrypy.response.status,
-                        'return': "Bad IP",
-                    }
+                    raise cherrypy.HTTPError(403, 'Bad IP')
 
 
 def salt_auth_tool():
@@ -535,9 +720,13 @@ def hypermedia_handler(*args, **kwargs):
     except (salt.exceptions.EauthAuthenticationError,
             salt.exceptions.TokenAuthenticationError):
         raise cherrypy.HTTPError(401)
+    except salt.exceptions.SaltInvocationError:
+        raise cherrypy.HTTPError(400)
     except (salt.exceptions.SaltDaemonNotRunning,
             salt.exceptions.SaltReqTimeoutError) as exc:
         raise cherrypy.HTTPError(503, exc.strerror)
+    except (cherrypy.TimeoutError, salt.exceptions.SaltClientTimeout):
+        raise cherrypy.HTTPError(504)
     except cherrypy.CherryPyException:
         raise
     except Exception as exc:
@@ -561,7 +750,12 @@ def hypermedia_handler(*args, **kwargs):
     # Transform the output from the handler into the requested output format
     cherrypy.response.headers['Content-Type'] = best
     out = cherrypy.response.processors[best]
-    return out(ret)
+    try:
+        return out(ret)
+    except Exception:
+        msg = 'Could not serialize the return data from Salt.'
+        logger.debug(msg, exc_info=True)
+        raise cherrypy.HTTPError(500, msg)
 
 
 def hypermedia_out():
@@ -708,7 +902,7 @@ def lowdata_fmt():
     # if the data was sent as urlencoded, we need to make it a list.
     # this is a very forgiving implementation as different clients set different
     # headers for form encoded data (including charset or something similar)
-    if data and not isinstance(data, list):
+    if data and isinstance(data, collections.Mapping):
         # Make the 'arg' param a list if not already
         if 'arg' in data and not isinstance(data['arg'], list):
             data['arg'] = [data['arg']]
@@ -723,12 +917,12 @@ cherrypy.tools.html_override = cherrypy.Tool('on_start_resource',
         html_override_tool, priority=53)
 cherrypy.tools.salt_token = cherrypy.Tool('on_start_resource',
         salt_token_tool, priority=55)
+cherrypy.tools.cors_tool = cherrypy.Tool('before_request_body',
+        cors_tool, priority=50)
 cherrypy.tools.salt_auth = cherrypy.Tool('before_request_body',
         salt_auth_tool, priority=60)
 cherrypy.tools.hypermedia_in = cherrypy.Tool('before_request_body',
         hypermedia_in)
-cherrypy.tools.cors_tool = cherrypy.Tool('before_request_body',
-        cors_tool, priority=30)
 cherrypy.tools.lowdata_fmt = cherrypy.Tool('before_handler',
         lowdata_fmt, priority=40)
 cherrypy.tools.hypermedia_out = cherrypy.Tool('before_handler',
@@ -841,14 +1035,9 @@ class LowDataAdapter(object):
         '''
         import inspect
 
-        # Grab all available client interfaces
-        clients = [name for name, _ in inspect.getmembers(salt.netapi.NetapiClient,
-            predicate=inspect.ismethod) if not name.startswith('__')]
-        clients.remove('run')  # run method calls client interfaces
-
         return {
             'return': "Welcome",
-            'clients': clients,
+            'clients': salt.netapi.CLIENTS,
         }
 
     @cherrypy.tools.salt_token()
@@ -866,6 +1055,7 @@ class LowDataAdapter(object):
             :resheader Content-Type: |res_ct|
 
             :status 200: |200|
+            :status 400: |400|
             :status 401: |401|
             :status 406: |406|
 
@@ -877,11 +1067,10 @@ class LowDataAdapter(object):
         .. code-block:: bash
 
             curl -sSik https://localhost:8000 \\
-                    -H "Accept: application/x-yaml" \\
-                    -H "X-Auth-Token: d40d1e1e<...snip...>" \\
-                    -d client=local \\
-                    -d tgt='*' \\
-                    -d fun='test.ping' \\
+                -b ~/cookies.txt \\
+                -H "Accept: application/x-yaml" \\
+                -H "Content-type: application/json" \\
+                -d '[{"client": "local", "tgt": "*", "fun": "test.ping"}]'
 
         .. code-block:: http
 
@@ -889,10 +1078,9 @@ class LowDataAdapter(object):
             Host: localhost:8000
             Accept: application/x-yaml
             X-Auth-Token: d40d1e1e
-            Content-Length: 36
-            Content-Type: application/x-www-form-urlencoded
+            Content-Type: application/json
 
-            fun=test.ping&client=local&tgt=*
+            [{"client": "local", "tgt": "*", "fun": "test.ping"}]
 
         **Example response:**
 
@@ -905,55 +1093,10 @@ class LowDataAdapter(object):
 
             return:
             - ms-0: true
-                ms-1: true
-                ms-2: true
-                ms-3: true
-                ms-4: true
-
-        **Other examples**:
-
-        .. code-block:: bash
-
-            # Sending multiple positional args with urlencoded:
-            curl -sSik https://localhost:8000 \\
-                    -d client=local \\
-                    -d tgt='*' \\
-                    -d fun='cmd.run' \\
-                    -d arg='du -sh .' \\
-                    -d arg='/path/to/dir'
-
-            # Sending positional args and Keyword args with JSON:
-            echo '[
-                {
-                    "client": "local",
-                    "tgt": "*",
-                    "fun": "cmd.run",
-                    "arg": [
-                        "du -sh .",
-                        "/path/to/dir"
-                    ],
-                    "kwarg": {
-                        "shell": "/bin/sh",
-                        "template": "jinja"
-                    }
-                }
-            ]' | curl -sSik https://localhost:8000 \\
-                    -H 'Content-type: application/json' \\
-                    -d@-
-
-            # Calling runner functions:
-            curl -sSik https://localhost:8000 \\
-                    -d client=runner \\
-                    -d fun='jobs.lookup_jid' \\
-                    -d jid='20150129182456704682' \\
-                    -d outputter=highstate
-
-            # Calling wheel functions:
-            curl -sSik https://localhost:8000 \\
-                    -d client=wheel \\
-                    -d fun='key.gen_accept' \\
-                    -d id_=dave \\
-                    -d keysize=4096
+              ms-1: true
+              ms-2: true
+              ms-3: true
+              ms-4: true
         '''
         return {
             'return': list(self.exec_lowstate(
@@ -1030,6 +1173,7 @@ class Minions(LowDataAdapter):
             :resheader Content-Type: |res_ct|
 
             :status 200: |200|
+            :status 400: |400|
             :status 401: |401|
             :status 406: |406|
 
@@ -1042,17 +1186,16 @@ class Minions(LowDataAdapter):
         .. code-block:: bash
 
             curl -sSi localhost:8000/minions \\
+                -b ~/cookies.txt \\
                 -H "Accept: application/x-yaml" \\
-                -d tgt='*' \\
-                -d fun='status.diskusage'
+                -d '[{"tgt": "*", "fun": "status.diskusage"}]'
 
         .. code-block:: http
 
             POST /minions HTTP/1.1
             Host: localhost:8000
             Accept: application/x-yaml
-            Content-Length: 26
-            Content-Type: application/x-www-form-urlencoded
+            Content-Type: application/json
 
             tgt=*&fun=status.diskusage
 
@@ -1379,10 +1522,10 @@ class Keys(LowDataAdapter):
         priv_key_file = tarfile.TarInfo('minion.pem')
         priv_key_file.size = len(priv_key)
 
-        fileobj = StringIO.StringIO()
+        fileobj = six.moves.StringIO()
         tarball = tarfile.open(fileobj=fileobj, mode='w')
-        tarball.addfile(pub_key_file, StringIO.StringIO(pub_key))
-        tarball.addfile(priv_key_file, StringIO.StringIO(priv_key))
+        tarball.addfile(pub_key_file, six.moves.StringIO(pub_key))
+        tarball.addfile(priv_key_file, six.moves.StringIO(priv_key))
         tarball.close()
 
         headers = cherrypy.response.headers
@@ -1468,20 +1611,25 @@ class Login(LowDataAdapter):
         .. code-block:: bash
 
             curl -si localhost:8000/login \\
-                    -H "Accept: application/json" \\
-                    -d username='saltuser' \\
-                    -d password='saltpass' \\
-                    -d eauth='pam'
+                -c ~/cookies.txt \\
+                -H "Accept: application/json" \\
+                -H "Content-type: application/json" \\
+                -d '{
+                    "username": "saltuser",
+                    "password": "saltuser",
+                    "eauth": "auto"
+                }'
 
         .. code-block:: http
 
             POST / HTTP/1.1
             Host: localhost:8000
             Content-Length: 42
-            Content-Type: application/x-www-form-urlencoded
+            Content-Type: application/json
             Accept: application/json
 
-            username=saltuser&password=saltpass&eauth=pam
+            {"username": "saltuser", "password": "saltuser", "eauth": "auto"}
+
 
         **Example response:**
 
@@ -1612,6 +1760,7 @@ class Run(LowDataAdapter):
             request body.
 
             :status 200: |200|
+            :status 400: |400|
             :status 401: |401|
             :status 406: |406|
 
@@ -1621,12 +1770,15 @@ class Run(LowDataAdapter):
 
             curl -sS localhost:8000/run \\
                 -H 'Accept: application/x-yaml' \\
-                -d client='local' \\
-                -d tgt='*' \\
-                -d fun='test.ping' \\
-                -d username='saltdev' \\
-                -d password='saltdev' \\
-                -d eauth='pam'
+                -H 'Content-type: application/json' \\
+                -d '[{
+                    "client": "local",
+                    "tgt": "*",
+                    "fun": "test.ping",
+                    "username": "saltdev",
+                    "password": "saltdev",
+                    "eauth": "auto"
+                }]'
 
         .. code-block:: http
 
@@ -1634,9 +1786,9 @@ class Run(LowDataAdapter):
             Host: localhost:8000
             Accept: application/x-yaml
             Content-Length: 75
-            Content-Type: application/x-www-form-urlencoded
+            Content-Type: application/json
 
-            client=local&tgt=*&fun=test.ping&username=saltdev&password=saltdev&eauth=pam
+            [{"client": "local", "tgt": "*", "fun": "test.ping", "username": "saltdev", "password": "saltdev", "eauth": "auto"}]
 
         **Example response:**
 
@@ -1648,17 +1800,19 @@ class Run(LowDataAdapter):
 
             return:
             - ms-0: true
-                ms-1: true
-                ms-2: true
-                ms-3: true
-                ms-4: true
+              ms-1: true
+              ms-2: true
+              ms-3: true
+              ms-4: true
 
-        The /run enpoint can also be used to issue commands using the salt-ssh subsystem.
+        The /run enpoint can also be used to issue commands using the salt-ssh
+        subsystem.
 
-        When using salt-ssh, eauth credentials should not be supplied. Instad, authentication
-        should be handled by the SSH layer itself. The use of the salt-ssh client does not
-        require a salt master to be running. Instead, only a roster file must be present
-        in the salt configuration directory.
+        When using salt-ssh, eauth credentials should not be supplied. Instad,
+        authentication should be handled by the SSH layer itself. The use of
+        the salt-ssh client does not require a salt master to be running.
+        Instead, only a roster file must be present in the salt configuration
+        directory.
 
         All SSH client requests are synchronous.
 
@@ -1745,9 +1899,9 @@ class Events(object):
 
         # First check if the given token is in our session table; if so it's a
         # salt-api token and we need to get the Salt token from there.
-        orig_sesion, _ = cherrypy.session.cache.get(auth_token, ({}, None))
+        orig_session, _ = cherrypy.session.cache.get(auth_token, ({}, None))
         # If it's not in the session table, assume it's a regular Salt token.
-        salt_token = orig_sesion.get('token', auth_token)
+        salt_token = orig_session.get('token', auth_token)
 
         # The eauth system does not currently support perms for the event
         # stream, so we're just checking if the token exists not if the token
@@ -1887,7 +2041,7 @@ class Events(object):
                     transport=self.opts['transport'],
                     opts=self.opts,
                     listen=True)
-            stream = event.iter_events(full=True)
+            stream = event.iter_events(full=True, auto_reconnect=True)
 
             yield u'retry: {0}\n'.format(400)
 
@@ -2032,8 +2186,8 @@ class WebsocketEndpoint(object):
         # Pulling the session token from an URL param is a workaround for
         # browsers not supporting CORS in the EventSource API.
         if token:
-            orig_sesion, _ = cherrypy.session.cache.get(token, ({}, None))
-            salt_token = orig_sesion.get('token')
+            orig_session, _ = cherrypy.session.cache.get(token, ({}, None))
+            salt_token = orig_session.get('token')
         else:
             salt_token = cherrypy.session.get('token')
 
@@ -2061,8 +2215,14 @@ class WebsocketEndpoint(object):
                     transport=self.opts['transport'],
                     opts=self.opts,
                     listen=True)
-            stream = event.iter_events(full=True)
+            stream = event.iter_events(full=True, auto_reconnect=True)
             SaltInfo = event_processor.SaltInfo(handler)
+
+            def signal_handler(signal, frame):
+                os._exit(0)
+
+            signal.signal(signal.SIGTERM, signal_handler)
+
             while True:
                 data = next(stream)
                 if data:
@@ -2171,16 +2331,18 @@ class Webhook(object):
 
         .. code-block:: bash
 
-            curl -sS localhost:8000/hook -d foo='Foo!' -d bar='Bar!'
+            curl -sS localhost:8000/hook \\
+                -H 'Content-type: application/json' \\
+                -d '{"foo": "Foo!", "bar": "Bar!"}'
 
         .. code-block:: http
 
             POST /hook HTTP/1.1
             Host: localhost:8000
             Content-Length: 16
-            Content-Type: application/x-www-form-urlencoded
+            Content-Type: application/json
 
-            foo=Foo&bar=Bar!
+            {"foo": "Foo!", "bar": "Bar!"}
 
         **Example response**:
 
@@ -2384,6 +2546,8 @@ class API(object):
                 'max_request_body_size': self.apiopts.get(
                     'max_request_body_size', 1048576),
                 'debug': self.apiopts.get('debug', False),
+                'log.access_file': self.apiopts.get('log_access_file', ''),
+                'log.error_file': self.apiopts.get('log_error_file', ''),
             },
             '/': {
                 'request.dispatch': cherrypy.dispatch.MethodDispatcher(),
