@@ -4,8 +4,8 @@ Use a git repository as a Pillar source
 ---------------------------------------
 
 .. note::
-    This external pillar has been rewritten for the :doc:`2015.8.0
-    </topics/releases/2015.8.0>` release. The old method of configuring this
+    This external pillar has been rewritten for the :ref:`2015.8.0
+    <release-2015-8-0>` release. The old method of configuring this
     external pillar will be maintained for a couple releases, allowing time for
     configurations to be updated to reflect the new usage.
 
@@ -40,12 +40,23 @@ the repo's URL. Configuration details can be found below.
           'dev-*':
             - bar
 
+    Additionally, while git_pillar allows for the branch/tag to be overridden
+    (see :ref:`here <git-pillar-env-remap>`, or :ref:`here
+    <git-pillar-env-remap-legacy>` for Salt releases before 2015.8.0), keep in
+    mind that the top file must reference the actual environment name. It is
+    common practice to make the environment in a git_pillar top file match the
+    branch/tag name, but when remapping, the environment of course no longer
+    matches the branch/tag, and the top file needs to be adjusted accordingly.
+    When expected Pillar values configured in git_pillar are missing, this is a
+    common misconfiguration that may be to blame, and is a good first step in
+    troubleshooting.
+
 .. _git-pillar-pre-2015-8-0:
 
 Configuring git_pillar for Salt releases before 2015.8.0
 ========================================================
 
-For Salt releases earlier than :doc:`2015.8.0 </topics/releases/2015.8.0>`,
+For Salt releases earlier than :ref:`2015.8.0 <release-2015-8-0>`,
 GitPython is the only supported provider for git_pillar. Individual
 repositories can be configured under the :conf_master:`ext_pillar`
 configuration parameter like so:
@@ -56,8 +67,8 @@ configuration parameter like so:
       - git: master https://gitserver/git-pillar.git root=subdirectory
 
 The repository is specified in the format ``<branch> <repo_url>``, with an
-optional ``root`` parameter (added in the :doc:`2014.7.0
-</topics/releases/2014.7.0>` release) which allows the pillar SLS files to be
+optional ``root`` parameter (added in the :ref:`2014.7.0
+<release-2014-7-0>` release) which allows the pillar SLS files to be
 served up from a subdirectory (similar to :conf_master:`gitfs_root` in gitfs).
 
 To use more than one branch from the same repo, multiple lines must be
@@ -69,6 +80,8 @@ specified under :conf_master:`ext_pillar`:
       - git: master https://gitserver/git-pillar.git
       - git: dev https://gitserver/git-pillar.git
 
+.. _git-pillar-env-remap-legacy:
+
 To remap a specific branch to a specific Pillar environment, use the format
 ``<branch>:<env>``:
 
@@ -96,8 +109,9 @@ it:
       '*':
         - bar
 
-If ``__env__`` is specified as the branch name, then git_pillar will use the
-branch specified by :conf_master:`gitfs_base`:
+If ``__env__`` is specified as the branch name, then git_pillar will first look
+at the minion's :conf_minion:`environment` option. If unset, it will fall back
+to using branch specified by the master's :conf_master:`gitfs_base`:
 
 .. code-block:: yaml
 
@@ -108,10 +122,27 @@ The corresponding Pillar top file would look like this:
 
 .. code-block:: yaml
 
-    {{env}}:
+    {{saltenv}}:
       '*':
         - bar
 
+.. note::
+    This feature was unintentionally omitted when git_pillar was rewritten for
+    the 2015.8.0 release. It was added again in the 2016.3.4 release, but it
+    has changed slightly in that release. On Salt masters running 2015.8.0
+    through 2016.3.3, this feature can only be accessed using the legacy config
+    described above. For 2016.3.4 and later, refer to explanation of the
+    ``__env__`` parameter in the below section.
+
+    Versions 2016.3.0 through 2016.3.4 incorrectly check the *master's*
+    ``environment`` config option (instead of the minion's) before falling back
+    to :conf_master:`gitfs_base`. This has been fixed in the 2016.3.5 and
+    2016.11.1 releases (2016.11.0 contains the incorrect behavior).
+
+    Additionally, in releases before 2016.11.0, both ``{{env}}`` and
+    ``{{saltenv}}`` could be used as a placeholder for the environment.
+    Starting in 2016.11.0, ``{{env}}`` is no longer supported.
+
 .. _git-pillar-2015-8-0-and-later:
 
 Configuring git_pillar for Salt releases 2015.8.0 and later
@@ -179,6 +210,77 @@ Per-remote configuration parameters are supported (similar to :ref:`gitfs
 <gitfs-per-remote-config>`), and global versions of the git_pillar
 configuration parameters can also be set.
 
+.. _git-pillar-env-remap:
+
+To remap a specific branch to a specific Pillar environment, use the ``env``
+per-remote parameter:
+
+.. code-block:: yaml
+
+    ext_pillar:
+      - git:
+        - production https://gitserver/git-pillar.git:
+          - env: prod
+
+If ``__env__`` is specified as the branch name, then git_pillar will decide
+which branch to use based on the following criteria:
+
+- If the minion has a :conf_minion:`pillarenv` configured, it will use that
+  pillar environment. (2016.11.2 and later)
+- Otherwise, if the minion has an ``environment`` configured, it will use that
+  environment.
+- Otherwise, the master's :conf_master:`git_pillar_base` will be used.
+
+.. note::
+    The use of :conf_minion:`environment` to choose the pillar environment
+    dates from a time before the :conf_minion:`pillarenv` parameter was added.
+    In a future release, it will be ignored and either the minion's
+    :conf_minion:`pillarenv` or the master's :conf_master:`git_pillar_base`
+    will be used.
+
+Here's an example of using ``__env__`` as the git_pillar environment:
+
+.. code-block:: yaml
+
+    ext_pillar:
+      - git:
+        - __env__ https://gitserver/git-pillar.git:
+          - root: pillar
+
+The corresponding Pillar top file would look like this:
+
+.. code-block:: yaml
+
+    {{saltenv}}:
+      '*':
+        - bar
+
+.. note::
+    This feature was unintentionally omitted when git_pillar was rewritten for
+    the 2015.8.0 release. It was added again in the 2016.3.4 release, but it
+    has changed slightly in that release. The fallback value replaced by
+    ``{{env}}`` is :conf_master: is :conf_master:`git_pillar_base`, while the
+    legacy config's version of this feature replaces ``{{env}}`` with
+    :conf_master:`gitfs_base`.
+
+    On Salt masters running 2015.8.0 through 2016.3.3, this feature can only be
+    accessed using the legacy config in the previous section of this page.
+
+    The same issue which affected the behavior of the minion's
+    :conf_minion:`environment` config value using the legacy configuration
+    syntax (see the documentation in the pre-2015.8.0 section above for the
+    legacy support of this feature) also affects the new-style git_pillar
+    syntax in version 2016.3.4. This has been corrected in version 2016.3.5 and
+    2016.11.1 (2016.11.0 contains the incorrect behavior).
+
+    2016.3.4 incorrectly checks the *master's* ``environment`` config option
+    (instead of the minion's) before falling back to the master's
+    :conf_master:`git_pillar_base`.
+
+    Additionally, in releases before 2016.11.0, both ``{{env}}`` and
+    ``{{saltenv}}`` could be used as a placeholder for the environment.
+    Starting in 2016.11.0, ``{{env}}`` is no longer supported.
+
 With the addition of pygit2_ support, git_pillar can now interact with
 authenticated remotes. Authentication works just like in gitfs (as outlined in
 the :ref:`Git Fileserver Backend Walkthrough <gitfs-authentication>`), only
@@ -187,10 +289,9 @@ instead of ``gitfs`` (e.g. :conf_master:`git_pillar_pubkey`,
 :conf_master:`git_pillar_privkey`, :conf_master:`git_pillar_passphrase`, etc.).
 
 .. note::
-
     The ``name`` parameter can be used to further differentiate between two
-    remotes with the same URL. If you're using two remotes with the same URL,
-    the ``name`` option is required.
+    remotes with the same URL and branch. When using two remotes with the same
+    URL, the ``name`` option is required.
 
 .. _GitPython: https://github.com/gitpython-developers/GitPython
 .. _pygit2: https://github.com/libgit2/pygit2
@@ -205,6 +306,7 @@ import hashlib
 import os
 
 # Import salt libs
+import salt.utils
 import salt.utils.gitfs
 import salt.utils.dictupdate
 from salt.exceptions import FileserverConfigError
@@ -301,6 +403,10 @@ def ext_pillar(minion_id, repo, pillar_dirs):
             # the pillar top.sls is sourced from the correct location.
             pillar_roots = [pillar_dir]
             pillar_roots.extend([x for x in all_dirs if x != pillar_dir])
+            if env == '__env__':
+                env = opts.get('pillarenv') \
+                    or opts.get('environment') \
+                    or opts.get('git_pillar_base')
             opts['pillar_roots'] = {env: pillar_roots}
 
             local_pillar = Pillar(opts, __grains__, minion_id, env)
@@ -330,9 +436,9 @@ class _LegacyGitPillar(object):
         self.working_dir = ''
         self.repo = None
 
-        hash_type = getattr(hashlib, opts.get('hash_type', 'md5'))
+        hash_type = getattr(hashlib, opts['hash_type'])
         hash_str = '{0} {1}'.format(self.branch, self.rp_location)
-        repo_hash = hash_type(hash_str).hexdigest()
+        repo_hash = hash_type(salt.utils.to_bytes(hash_str)).hexdigest()
         rp_ = os.path.join(self.opts['cachedir'], 'pillar_gitfs', repo_hash)
 
         if not os.path.isdir(rp_):
@@ -388,7 +494,7 @@ class _LegacyGitPillar(object):
         Return boolean whether it worked
         '''
         try:
-            log.debug('Updating fileserver for git_pillar module')
+            log.debug('Legacy git_pillar: Updating \'%s\'', self.rp_location)
             self.repo.git.fetch()
         except git.exc.GitCommandError as exc:
             log.error('Unable to fetch the latest changes from remote '
@@ -396,10 +502,15 @@ class _LegacyGitPillar(object):
             return False
 
         try:
-            self.repo.git.checkout('origin/{0}'.format(self.branch))
+            checkout_ref = 'origin/{0}'.format(self.branch)
+            log.debug('Legacy git_pillar: Checking out %s for \'%s\'',
+                      checkout_ref, self.rp_location)
+            self.repo.git.checkout(checkout_ref)
         except git.exc.GitCommandError as exc:
-            log.error('Unable to checkout branch '
-                      '{0}: {1}'.format(self.branch, exc))
+            log.error(
+                'Legacy git_pillar: Failed to checkout %s for \'%s\': %s',
+                checkout_ref, self.rp_location, exc
+            )
             return False
 
         return True
@@ -440,13 +551,20 @@ def _legacy_git_pillar(minion_id, repo_string, pillar_dirs):
         # Support multiple key=val attributes as custom parameters.
         DELIM = '='
         if DELIM not in extraopt:
-            log.error('Incorrectly formatted extra parameter. '
-                      'Missing \'{0}\': {1}'.format(DELIM, extraopt))
+            log.error(
+                'Legacy git_pillar: Incorrectly formatted extra parameter '
+                '\'%s\' within \'%s\' missing \'%s\')',
+                extraopt, repo_string, DELIM
+            )
         key, val = _extract_key_val(extraopt, DELIM)
         if key == 'root':
             root = val
         else:
-            log.warning('Unrecognized extra parameter: {0}'.format(key))
+            log.error(
+                'Legacy git_pillar: Unrecognized extra parameter \'%s\' '
+                'in \'%s\'',
+                key, repo_string
+            )
 
     # environment is "different" from the branch
     cfg_branch, _, environment = branch_env.partition(':')
@@ -462,12 +580,24 @@ def _legacy_git_pillar(minion_id, repo_string, pillar_dirs):
 
     # normpath is needed to remove appended '/' if root is empty string.
     pillar_dir = os.path.normpath(os.path.join(gitpil.working_dir, root))
+    log.debug(
+        'Legacy git_pillar: pillar_dir for \'%s\' is \'%s\'',
+        repo_string, pillar_dir
+    )
+    log.debug(
+        'Legacy git_pillar: branch for \'%s\' is \'%s\'',
+        repo_string, branch
+    )
 
     pillar_dirs.setdefault(pillar_dir, {})
 
     if cfg_branch == '__env__' and branch not in ['master', 'base']:
         gitpil.update()
     elif pillar_dirs[pillar_dir].get(branch, False):
+        log.debug(
+            'Already processed pillar_dir \'%s\' for \'%s\'',
+            pillar_dir, repo_string
+        )
         return {}  # we've already seen this combo
 
     pillar_dirs[pillar_dir].setdefault(branch, True)
