@@ -634,59 +634,6 @@ def stopped(name, kill=False, path=None):
     return ret
 
 
-# Deprecated states
-def created(name, **kwargs):
-    '''
-    .. deprecated:: 2015.5.0
-        Use :mod:`lxc.present <salt.states.lxc.present>`
-    '''
-    salt.utils.warn_until(
-        'Boron',
-        'The lxc.created state has been renamed to lxc.present, please use '
-        'lxc.present'
-    )
-    return present(name, **kwargs)
-
-
-def started(name, path=None, restart=False):
-    '''
-    .. deprecated:: 2015.5.0
-        Use :mod:`lxc.running <salt.states.lxc.running>`
-    '''
-    salt.utils.warn_until(
-        'Boron',
-        'The lxc.started state has been renamed to lxc.running, please use '
-        'lxc.running'
-    )
-    return running(name, restart=restart, path=path)
-
-
-def cloned(name,
-           orig,
-           snapshot=True,
-           size=None,
-           vgname=None,
-           path=None,
-           profile=None):
-    '''
-    .. deprecated:: 2015.5.0
-        Use :mod:`lxc.present <salt.states.lxc.present>`
-    '''
-    salt.utils.warn_until(
-        'Boron',
-        'The lxc.cloned state has been merged into the lxc.present state. '
-        'Please update your states to use lxc.present, with the '
-        '\'clone_from\' argument set to the name of the clone source.'
-    )
-    return present(name,
-                   clone_from=orig,
-                   snapshot=snapshot,
-                   size=size,
-                   vgname=vgname,
-                   path=path,
-                   profile=profile)
-
-
 def set_pass(name, **kwargs):  # pylint: disable=W0613
     '''
     .. deprecated:: 2015.5.0
@@ -726,6 +673,8 @@ def edited_conf(name, lxc_conf=None, lxc_conf_unset=None):
 
     Edit LXC configuration options
 
+    .. deprecated:: 2015.5.0
+
     path
         path to the container parent
         default: /var/lib/lxc (system default)
@@ -743,8 +692,11 @@ def edited_conf(name, lxc_conf=None, lxc_conf_unset=None):
             - lxc_conf_unset:
                 - lxc.utsname
     '''
+    # Until a reasonable alternative for this state function is created, we need
+    # to keep this function around and cannot officially remove it. Progress of
+    # the new function will be tracked in https://github.com/saltstack/salt/issues/35523
     salt.utils.warn_until(
-        'Boron',
+        'Oxygen',
         'This state is unsuitable for setting parameters that appear more '
         'than once in an LXC config file, or parameters which must appear in '
         'a certain order (such as when configuring more than one network '
