@@ -87,8 +87,8 @@ def creds(provider):
                 proxies={'http': ''}, timeout=AWS_METADATA_TIMEOUT,
             )
             result.raise_for_status()
-            role = result.text
-        except (requests.exceptions.HTTPError, requests.exceptions.ConnectTimeout):
+            role = result.text.encode(result.encoding)
+        except (requests.exceptions.HTTPError, requests.exceptions.ConnectionError):
             return provider['id'], provider['key'], ''
 
         try:
@@ -97,7 +97,7 @@ def creds(provider):
                 proxies={'http': ''}, timeout=AWS_METADATA_TIMEOUT,
             )
             result.raise_for_status()
-        except (requests.exceptions.HTTPError, requests.exceptions.ConnectTimeout):
+        except (requests.exceptions.HTTPError, requests.exceptions.ConnectionError):
             return provider['id'], provider['key'], ''
 
         data = result.json()
@@ -203,7 +203,8 @@ def assumed_creds(prov_dict, role_arn, location=None):
 
 def sig4(method, endpoint, params, prov_dict,
          aws_api_version=DEFAULT_AWS_API_VERSION, location=None,
-         product='ec2', uri='/', requesturl=None, data='', role_arn=None):
+         product='ec2', uri='/', requesturl=None, data='', headers=None,
+         role_arn=None, payload_hash=None):
     '''
     Sign a query against AWS services using Signature Version 4 Signing
     Process. This is documented at:
@@ -226,7 +227,7 @@ def sig4(method, endpoint, params, prov_dict,
         location = DEFAULT_LOCATION
 
     params_with_headers = params.copy()
-    if product != 's3':
+    if product not in ('s3', 'ssm'):
         params_with_headers['Version'] = aws_api_version
     keys = sorted(params_with_headers.keys())
     values = list(map(params_with_headers.get, keys))
@@ -235,13 +236,19 @@ def sig4(method, endpoint, params, prov_dict,
     amzdate = timenow.strftime('%Y%m%dT%H%M%SZ')
     datestamp = timenow.strftime('%Y%m%d')
 
-    canonical_headers = 'host:{0}\nx-amz-date:{1}\n'.format(
+    canonical_headers = 'host:{0}\nx-amz-date:{1}'.format(
         endpoint,
         amzdate,
     )
 
     signed_headers = 'host;x-amz-date'
 
+    if isinstance(headers, dict):
+        for header in sorted(headers.keys()):
+            canonical_headers += '\n{0}:{1}'.format(header, headers[header])
+            signed_headers += ';{0}'.format(header)
+    canonical_headers += '\n'
+
     if token != '':
         canonical_headers += 'x-amz-security-token:{0}\n'.format(token)
         signed_headers += ';x-amz-security-token'
@@ -250,7 +257,8 @@ def sig4(method, endpoint, params, prov_dict,
 
     # Create payload hash (hash of the request body content). For GET
     # requests, the payload is an empty string ('').
-    payload_hash = hashlib.sha256(data).hexdigest()
+    if not payload_hash:
+        payload_hash = hashlib.sha256(data).hexdigest()
 
     # Combine elements to create create canonical request
     canonical_request = '\n'.join((
@@ -296,18 +304,21 @@ def sig4(method, endpoint, params, prov_dict,
             signature,
         )
 
-    headers = {
+    new_headers = {
         'x-amz-date': amzdate,
         'x-amz-content-sha256': payload_hash,
         'Authorization': authorization_header,
     }
+    if isinstance(headers, dict):
+        for header in sorted(headers.keys()):
+            new_headers[header] = headers[header]
 
     # Add in security token if we have one
     if token != '':
-        headers['X-Amz-Security-Token'] = token
+        new_headers['X-Amz-Security-Token'] = token
 
     requesturl = '{0}?{1}'.format(requesturl, querystring)
-    return headers, requesturl
+    return new_headers, requesturl
 
 
 def _sign(key, msg):
@@ -420,6 +431,11 @@ def query(params=None, setname=None, requesturl=None, location=None,
         )
     )
 
+    # Fallback to ec2's id & key if none is found, for this component
+    if not prov_dict.get('id', None):
+        prov_dict['id'] = providers.get(provider, {}).get('ec2', {}).get('id', {})
+        prov_dict['key'] = providers.get(provider, {}).get('ec2', {}).get('key', {})
+
     if sigver == '4':
         headers, requesturl = sig4(
             method, endpoint, params, prov_dict, aws_api_version, location, product, requesturl=requesturl
@@ -444,7 +460,7 @@ def query(params=None, setname=None, requesturl=None, location=None,
             )
             LOG.trace(
                 'AWS Response Text: {0}'.format(
-                    result.text
+                    result.text.encode(result.encoding)
                 )
             )
             result.raise_for_status()
@@ -485,7 +501,7 @@ def query(params=None, setname=None, requesturl=None, location=None,
             return {'error': data}, requesturl
         return {'error': data}
 
-    response = result.text
+    response = result.text.encode(result.encoding)
 
     root = ET.fromstring(response)
     items = root[1]
@@ -552,7 +568,7 @@ def get_region_from_metadata():
     return None
 
 
-def get_location(opts, provider=None):
+def get_location(opts=None, provider=None):
     '''
     Return the region to use, in this order:
         opts['location']
@@ -560,7 +576,11 @@ def get_location(opts, provider=None):
         get_region_from_metadata()
         DEFAULT_LOCATION
     '''
-    ret = opts.get('location', provider.get('location'))
+    if opts is None:
+        opts = {}
+    ret = opts.get('location')
+    if ret is None and provider is not None:
+        ret = provider.get('location')
     if ret is None:
         ret = get_region_from_metadata()
     if ret is None:
