@@ -15,8 +15,8 @@ file or for the minion via config or pillar)
         kwargs:
           test: True
 
-This will schedule the command: state.sls httpd test=True every 3600 seconds
-(every hour)
+This will schedule the command: ``state.sls httpd test=True`` every 3600 seconds
+(every hour).
 
 .. code-block:: yaml
 
@@ -30,8 +30,8 @@ This will schedule the command: state.sls httpd test=True every 3600 seconds
           test: True
         splay: 15
 
-This will schedule the command: state.sls httpd test=True every 3600 seconds
-(every hour) splaying the time between 0 and 15 seconds
+This will schedule the command: ``state.sls httpd test=True`` every 3600 seconds
+(every hour) splaying the time between 0 and 15 seconds.
 
 .. code-block:: yaml
 
@@ -47,13 +47,14 @@ This will schedule the command: state.sls httpd test=True every 3600 seconds
           start: 10
           end: 15
 
-This will schedule the command: state.sls httpd test=True every 3600 seconds
-(every hour) splaying the time between 10 and 15 seconds
+This will schedule the command: ``state.sls httpd test=True`` every 3600 seconds
+(every hour) splaying the time between 10 and 15 seconds.
 
 .. versionadded:: 2014.7.0
 
 Frequency of jobs can also be specified using date strings supported by
-the python dateutil library.
+the Python ``dateutil`` library. This requires the Python ``dateutil`` library
+to be installed.
 
 .. code-block:: yaml
 
@@ -66,7 +67,7 @@ the python dateutil library.
           test: True
         when: 5:00pm
 
-This will schedule the command: state.sls httpd test=True at 5:00pm minion
+This will schedule the command: ``state.sls httpd test=True`` at 5:00 PM minion
 localtime.
 
 .. code-block:: yaml
@@ -79,15 +80,69 @@ localtime.
         kwargs:
           test: True
         when:
-            - Monday 5:00pm
-            - Tuesday 3:00pm
-            - Wednesday 5:00pm
-            - Thursday 3:00pm
-            - Friday 5:00pm
+          - Monday 5:00pm
+          - Tuesday 3:00pm
+          - Wednesday 5:00pm
+          - Thursday 3:00pm
+          - Friday 5:00pm
 
-This will schedule a job to run once on the specified date. The default date
-format is ISO 8601 but can be overridden by also specifying the ``once_fmt``
-option.
+This will schedule the command: ``state.sls httpd test=True`` at 5:00 PM on
+Monday, Wednesday and Friday, and 3:00 PM on Tuesday and Thursday.
+
+.. code-block:: yaml
+
+    schedule:
+      job1:
+        function: state.sls
+        seconds: 3600
+        args:
+          - httpd
+        kwargs:
+          test: True
+        range:
+          start: 8:00am
+          end: 5:00pm
+
+This will schedule the command: ``state.sls httpd test=True`` every 3600 seconds
+(every hour) between the hours of 8:00 AM and 5:00 PM. The range parameter must
+be a dictionary with the date strings using the ``dateutil`` format.
+
+.. code-block:: yaml
+
+    schedule:
+      job1:
+        function: state.sls
+        seconds: 3600
+        args:
+          - httpd
+        kwargs:
+          test: True
+        range:
+          invert: True
+          start: 8:00am
+          end: 5:00pm
+
+Using the invert option for range, this will schedule the command
+``state.sls httpd test=True`` every 3600 seconds (every hour) until the current
+time is between the hours of 8:00 AM and 5:00 PM. The range parameter must be
+a dictionary with the date strings using the ``dateutil`` format.
+
+.. code-block:: yaml
+
+    schedule:
+      job1:
+        function: pkg.install
+        kwargs:
+          pkgs: [{'bar': '>1.2.3'}]
+          refresh: true
+        once: '2016-01-07T14:30:00'
+
+This will schedule the function ``pkg.install`` to be executed once at the
+specified time. The schedule entry ``job1`` will not be removed after the job
+completes, therefore use ``schedule.delete`` to manually remove it afterwards.
+
+The default date format is ISO 8601 but can be overridden by also specifying the
+``once_fmt`` option, like this:
 
 .. code-block:: yaml
 
@@ -97,67 +152,21 @@ option.
         once: 2015-04-22T20:21:00
         once_fmt: '%Y-%m-%dT%H:%M:%S'
 
-This will schedule the command: state.sls httpd test=True at 5pm on Monday,
-Wednesday and Friday, and 3pm on Tuesday and Thursday.
-
-.. code-block:: yaml
-
-    schedule:
-      job1:
-        function: state.sls
-        seconds: 3600
-        args:
-          - httpd
-        kwargs:
-          test: True
-        range:
-            start: 8:00am
-            end: 5:00pm
-
-This will schedule the command: state.sls httpd test=True every 3600 seconds
-(every hour) between the hours of 8am and 5pm.  The range parameter must be a
-dictionary with the date strings using the dateutil format.
-
 .. versionadded:: 2014.7.0
 
-.. code-block:: yaml
+The scheduler also supports ensuring that there are no more than N copies of
+a particular routine running.  Use this for jobs that may be long-running
+and could step on each other or pile up in case of infrastructure outage.
 
-    schedule:
-      job1:
-        function: state.sls
-        seconds: 3600
-        args:
-          - httpd
-        kwargs:
-          test: True
-        range:
-            invert: True
-            start: 8:00am
-            end: 5:00pm
-
-Using the invert option for range, this will schedule the command: state.sls
-httpd test=True every 3600 seconds (every hour) until the current time is
-between the hours of 8am and 5pm.  The range parameter must be a dictionary
-with the date strings using the dateutil format.
-
-By default any job scheduled based on the startup time of the minion will run
-the scheduled job when the minion starts up.  Sometimes this is not the desired
-situation.  Using the 'run_on_start' parameter set to False will cause the
-scheduler to skip this first run and wait until the next scheduled run.
-
-.. versionadded:: 2015.5.0
+The default for ``maxrunning`` is 1.
 
 .. code-block:: yaml
 
     schedule:
-      job1:
-        function: state.sls
-        seconds: 3600
-        run_on_start: False
-        args:
-          - httpd
-        kwargs:
-          test: True
+      long_running_job:
+        function: big_file_transfer
+        jid_include: True
+        maxrunning: 1
 
 .. versionadded:: 2014.7.0
 
@@ -173,9 +182,60 @@ scheduler to skip this first run and wait until the next scheduled run.
           test: True
 
 The scheduler also supports scheduling jobs using a cron like format.
-This requires the python-croniter library.
+This requires the Python ``croniter`` library.
 
-    ... versionadded:: 2015.8.0
+.. versionadded:: 2015.5.0
+
+By default, data about jobs runs from the Salt scheduler is returned to the
+master. Setting the ``return_job`` parameter to False will prevent the data
+from being sent back to the Salt master.
+
+.. code-block:: yaml
+
+    schedule:
+      job1:
+        function: scheduled_job_function
+        return_job: False
+
+.. versionadded:: 2015.5.0
+
+It can be useful to include specific data to differentiate a job from other
+jobs. Using the metadata parameter special values can be associated with
+a scheduled job. These values are not used in the execution of the job,
+but can be used to search for specific jobs later if combined with the
+``return_job`` parameter. The metadata parameter must be specified as a
+dictionary, othewise it will be ignored.
+
+.. code-block:: yaml
+
+    schedule:
+      job1:
+        function: scheduled_job_function
+        metadata:
+          foo: bar
+
+.. versionadded:: 2015.5.0
+
+By default any job scheduled based on the startup time of the minion will run
+the scheduled job when the minion starts up. Sometimes this is not the desired
+situation. Using the ``run_on_start`` parameter set to ``False`` will cause the
+scheduler to skip this first run and wait until the next scheduled run:
+
+.. code-block:: yaml
+
+    schedule:
+      job1:
+        function: state.sls
+        seconds: 3600
+        run_on_start: False
+        args:
+          - httpd
+        kwargs:
+          test: True
+
+.. versionadded:: 2015.8.0
+
+.. code-block:: yaml
 
     schedule:
       job1:
@@ -188,12 +248,14 @@ This requires the python-croniter library.
           test: True
 
 Using the until argument, the Salt scheduler allows you to specify
-an end time for a scheduled job.  If this argument is specified, jobs
-will not run once the specified time has passed.  Time should be specified
-in a format support by the dateutil library.
-This requires the python-dateutil library.
+an end time for a scheduled job. If this argument is specified, jobs
+will not run once the specified time has passed. Time should be specified
+in a format supported by the ``dateutil`` library.
+This requires the Python ``dateutil`` library to be installed.
 
-    ... versionadded:: 2015.8.0
+.. versionadded:: 2015.8.0
+
+.. code-block:: yaml
 
     schedule:
       job1:
@@ -207,9 +269,9 @@ This requires the python-dateutil library.
 
 Using the after argument, the Salt scheduler allows you to specify
 an start time for a scheduled job.  If this argument is specified, jobs
-will not run until the specified time has passed.  Time should be specified
-in a format support by the dateutil library.
-This requires the python-dateutil library.
+will not run until the specified time has passed. Time should be specified
+in a format supported by the ``dateutil`` library.
+This requires the Python ``dateutil`` library to be installed.
 
 The scheduler also supports ensuring that there are no more than N copies of
 a particular routine running.  Use this for jobs that may be long-running
@@ -236,6 +298,10 @@ from being sent back to the Salt master.
           function: scheduled_job_function
           return_job: False
 
+Setting the ``return_job`` parameter to 'nocache' prevents the salt master
+from storing the job in the master cache. Still, an event is fired on the
+master event bus in the form 'salt/job/nocache/ret/myminion'.
+
 It can be useful to include specific data to differentiate a job from other
 jobs.  Using the metadata parameter special values can be associated with
 a scheduled job.  These values are not used in the execution of the job,
@@ -254,31 +320,37 @@ dictionary, othewise it will be ignored.
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, with_statement
 import os
+import sys
 import time
+import signal
 import datetime
 import itertools
-import multiprocessing
 import threading
 import logging
 import errno
 import random
+import yaml
 
 # Import Salt libs
+import salt.config
 import salt.utils
 import salt.utils.jid
 import salt.utils.process
 import salt.utils.args
+import salt.utils.minion
 import salt.loader
 import salt.minion
 import salt.payload
 import salt.syspaths
+import salt.exceptions
+import salt.log.setup as log_setup
+import salt.defaults.exitcodes
 from salt.utils.odict import OrderedDict
-from salt.utils.process import os_is_running
+from salt.utils.process import os_is_running, default_signals, SignalHandlingMultiprocessingProcess
 
 # Import 3rd-party libs
-import yaml
 import salt.ext.six as six
 
 # pylint: disable=import-error
@@ -304,7 +376,29 @@ class Schedule(object):
     '''
     Create a Schedule object, pass in the opts and the functions dict to use
     '''
-    def __init__(self, opts, functions, returners=None, intervals=None):
+    instance = None
+
+    def __new__(cls, opts, functions, returners=None, intervals=None, cleanup=None):
+        '''
+        Only create one instance of Schedule
+        '''
+        if cls.instance is None:
+            log.debug('Initializing new Schedule')
+            # we need to make a local variable for this, as we are going to store
+            # it in a WeakValueDictionary-- which will remove the item if no one
+            # references it-- this forces a reference while we return to the caller
+            cls.instance = object.__new__(cls)
+            cls.instance.__singleton_init__(opts, functions, returners, intervals, cleanup)
+        else:
+            log.debug('Re-using Schedule')
+        return cls.instance
+
+    # has to remain empty for singletons, since __init__ will *always* be called
+    def __init__(self, opts, functions, returners=None, intervals=None, cleanup=None):
+        pass
+
+    # an init for the singleton instance to call
+    def __singleton_init__(self, opts, functions, returners=None, intervals=None, cleanup=None):
         self.opts = opts
         self.functions = functions
         if isinstance(intervals, dict):
@@ -320,6 +414,12 @@ class Schedule(object):
         # Keep track of the lowest loop interval needed in this variable
         self.loop_interval = six.MAXSIZE
         clean_proc_dir(opts)
+        if cleanup:
+            for prefix in cleanup:
+                self.delete_job_prefix(prefix)
+
+    def __getnewargs__(self):
+        return self.opts, self.functions, self.returners, self.intervals, None
 
     def option(self, opt):
         '''
@@ -333,16 +433,32 @@ class Schedule(object):
         '''
         Persist the modified schedule into <<configdir>>/minion.d/_schedule.conf
         '''
-        schedule_conf = os.path.join(
-                salt.syspaths.CONFIG_DIR,
-                'minion.d',
-                '_schedule.conf')
+        config_dir = self.opts.get('conf_dir', None)
+        if config_dir is None and 'conf_file' in self.opts:
+            config_dir = os.path.dirname(self.opts['conf_file'])
+        if config_dir is None:
+            config_dir = salt.syspaths.CONFIG_DIR
+
+        minion_d_dir = os.path.join(
+            config_dir,
+            os.path.dirname(self.opts.get('default_include',
+                                          salt.config.DEFAULT_MINION_OPTS['default_include'])))
+
+        if not os.path.isdir(minion_d_dir):
+            os.makedirs(minion_d_dir)
+
+        schedule_conf = os.path.join(minion_d_dir, '_schedule.conf')
         log.debug('Persisting schedule')
         try:
             with salt.utils.fopen(schedule_conf, 'wb+') as fp_:
-                fp_.write(yaml.dump({'schedule': self.opts['schedule']}))
+                fp_.write(
+                    salt.utils.to_bytes(
+                        yaml.dump({'schedule': self.option('schedule')})
+                    )
+                )
         except (IOError, OSError):
-            log.error('Failed to persist the updated schedule')
+            log.error('Failed to persist the updated schedule',
+                      exc_info_on_loglevel=logging.DEBUG)
 
     def delete_job(self, name, persist=True, where=None):
         '''
@@ -350,16 +466,16 @@ class Schedule(object):
         '''
         if where is None or where != 'pillar':
             # ensure job exists, then delete it
-            if name in self.opts['schedule']:
-                del self.opts['schedule'][name]
-            schedule = self.opts['schedule']
+            schedule = self.option('schedule')
+            if name in schedule:
+                del schedule[name]
         else:
             # If job is in pillar, delete it there too
             if 'schedule' in self.opts['pillar']:
                 if name in self.opts['pillar']['schedule']:
                     del self.opts['pillar']['schedule'][name]
             schedule = self.opts['pillar']['schedule']
-            log.warn('Pillar schedule deleted. Pillar refresh recommended. Run saltutil.refresh_pillar.')
+            log.warning('Pillar schedule deleted. Pillar refresh recommended. Run saltutil.refresh_pillar.')
 
         # Fire the complete event back along with updated list of schedule
         evt = salt.utils.event.get_event('minion', opts=self.opts, listen=False)
@@ -373,6 +489,38 @@ class Schedule(object):
         if persist:
             self.persist()
 
+    def delete_job_prefix(self, name, persist=True, where=None):
+        '''
+        Deletes a job from the scheduler.
+        '''
+        if where is None or where != 'pillar':
+            # ensure job exists, then delete it
+            schedule = self.option('schedule')
+            for job in list(schedule.keys()):
+                if job.startswith(name):
+                    del schedule[job]
+        else:
+            # If job is in pillar, delete it there too
+            if 'schedule' in self.opts['pillar']:
+                for job in list(self.opts['pillar']['schedule'].keys()):
+                    if job.startswith(name):
+                        del self.opts['pillar']['schedule'][job]
+            schedule = self.opts['pillar']['schedule']
+            log.warning('Pillar schedule deleted. Pillar refresh recommended. Run saltutil.refresh_pillar.')
+
+        # Fire the complete event back along with updated list of schedule
+        evt = salt.utils.event.get_event('minion', opts=self.opts, listen=False)
+        evt.fire_event({'complete': True, 'schedule': schedule},
+                       tag='/salt/minion/minion_schedule_delete_complete')
+
+        # remove from self.intervals
+        for job in list(self.intervals.keys()):
+            if job.startswith(name):
+                del self.intervals[job]
+
+        if persist:
+            self.persist()
+
     def add_job(self, data, persist=True):
         '''
         Adds a new job to the scheduler. The format is the same as required in
@@ -387,19 +535,26 @@ class Schedule(object):
         if not len(data) == 1:
             raise ValueError('You can only schedule one new job at a time.')
 
+        # if enabled is not included in the job,
+        # assume job is enabled.
+        for job in data.keys():
+            if 'enabled' not in data[job]:
+                data[job]['enabled'] = True
+
         new_job = next(six.iterkeys(data))
 
-        if new_job in self.opts['schedule']:
+        schedule = self.option('schedule')
+        if new_job in schedule:
             log.info('Updating job settings for scheduled '
                      'job: {0}'.format(new_job))
         else:
             log.info('Added new job {0} to scheduler'.format(new_job))
 
-        self.opts['schedule'].update(data)
+        schedule.update(data)
 
         # Fire the complete event back along with updated list of schedule
         evt = salt.utils.event.get_event('minion', opts=self.opts, listen=False)
-        evt.fire_event({'complete': True, 'schedule': self.opts['schedule']},
+        evt.fire_event({'complete': True, 'schedule': schedule},
                        tag='/salt/minion/minion_schedule_add_complete')
 
         if persist:
@@ -413,8 +568,8 @@ class Schedule(object):
             self.opts['pillar']['schedule'][name]['enabled'] = True
             schedule = self.opts['pillar']['schedule']
         else:
-            self.opts['schedule'][name]['enabled'] = True
-            schedule = self.opts['schedule']
+            schedule = self.option('schedule')
+            schedule[name]['enabled'] = True
 
         # Fire the complete event back along with updated list of schedule
         evt = salt.utils.event.get_event('minion', opts=self.opts, listen=False)
@@ -434,8 +589,8 @@ class Schedule(object):
             self.opts['pillar']['schedule'][name]['enabled'] = False
             schedule = self.opts['pillar']['schedule']
         else:
-            self.opts['schedule'][name]['enabled'] = False
-            schedule = self.opts['schedule']
+            schedule = self.option('schedule')
+            schedule[name]['enabled'] = False
 
         # Fire the complete event back along with updated list of schedule
         evt = salt.utils.event.get_event('minion', opts=self.opts, listen=False)
@@ -456,9 +611,9 @@ class Schedule(object):
                 self.delete_job(name, persist, where=where)
             self.opts['pillar']['schedule'][name] = schedule
         else:
-            if name in self.opts['schedule']:
+            if name in self.option('schedule'):
                 self.delete_job(name, persist, where=where)
-            self.opts['schedule'][name] = schedule
+            self.option('schedule')[name] = schedule
 
         if persist:
             self.persist()
@@ -467,7 +622,7 @@ class Schedule(object):
         '''
         Run a schedule job now
         '''
-        schedule = self.opts['schedule']
+        schedule = self.option('schedule')
         if 'schedule' in self.opts['pillar']:
             schedule.update(self.opts['pillar']['schedule'])
         data = schedule[name]
@@ -492,52 +647,58 @@ class Schedule(object):
         log.info(
             'Running Job: {0}.'.format(name)
         )
-        if self.opts.get('multiprocessing', True):
-            thread_cls = multiprocessing.Process
+
+        multiprocessing_enabled = self.opts.get('multiprocessing', True)
+        if multiprocessing_enabled:
+            thread_cls = SignalHandlingMultiprocessingProcess
         else:
             thread_cls = threading.Thread
-        proc = thread_cls(target=self.handle_func, args=(func, data))
-        proc.start()
-        if self.opts.get('multiprocessing', True):
+
+        if multiprocessing_enabled:
+            with default_signals(signal.SIGINT, signal.SIGTERM):
+                proc = thread_cls(target=self.handle_func, args=(multiprocessing_enabled, func, data))
+                # Reset current signals before starting the process in
+                # order not to inherit the current signal handlers
+                proc.start()
             proc.join()
+        else:
+            proc = thread_cls(target=self.handle_func, args=(multiprocessing_enabled, func, data))
+            proc.start()
 
     def enable_schedule(self):
         '''
         Enable the scheduler.
         '''
-        self.opts['schedule']['enabled'] = True
+        schedule = self.option('schedule')
+        schedule['enabled'] = True
 
         # Fire the complete event back along with updated list of schedule
         evt = salt.utils.event.get_event('minion', opts=self.opts, listen=False)
-        evt.fire_event({'complete': True, 'schedule': self.opts['schedule']},
+        evt.fire_event({'complete': True, 'schedule': schedule},
                        tag='/salt/minion/minion_schedule_enabled_complete')
 
     def disable_schedule(self):
         '''
         Disable the scheduler.
         '''
-        self.opts['schedule']['enabled'] = False
+        schedule = self.option('schedule')
+        schedule['enabled'] = False
 
         # Fire the complete event back along with updated list of schedule
         evt = salt.utils.event.get_event('minion', opts=self.opts, listen=False)
-        evt.fire_event({'complete': True, 'schedule': self.opts['schedule']},
+        evt.fire_event({'complete': True, 'schedule': schedule},
                        tag='/salt/minion/minion_schedule_disabled_complete')
 
     def reload(self, schedule):
         '''
         Reload the schedule from saved schedule file.
         '''
-
         # Remove all jobs from self.intervals
         self.intervals = {}
 
-        if 'schedule' in self.opts:
-            if 'schedule' in schedule:
-                self.opts['schedule'].update(schedule['schedule'])
-            else:
-                self.opts['schedule'].update(schedule)
-        else:
-            self.opts['schedule'] = schedule
+        if 'schedule' in schedule:
+            schedule = schedule['schedule']
+        self.opts.setdefault('schedule', {}).update(schedule)
 
     def list(self, where):
         '''
@@ -548,9 +709,9 @@ class Schedule(object):
             if 'schedule' in self.opts['pillar']:
                 schedule.update(self.opts['pillar']['schedule'])
         elif where == 'opts':
-            schedule.update(self.opts['schedule'])
+            schedule.update(self.option('schedule'))
         else:
-            schedule.update(self.opts['schedule'])
+            schedule.update(self.option('schedule'))
             if 'schedule' in self.opts['pillar']:
                 schedule.update(self.opts['pillar']['schedule'])
 
@@ -570,18 +731,22 @@ class Schedule(object):
         evt.fire_event({'complete': True},
                        tag='/salt/minion/minion_schedule_saved')
 
-    def handle_func(self, func, data):
+    def handle_func(self, multiprocessing_enabled, func, data):
         '''
         Execute this method in a multiprocess or thread
         '''
-        if salt.utils.is_windows():
+        if salt.utils.is_windows() or self.opts.get('transport') == 'zeromq':
             # Since function references can't be pickled and pickling
             # is required when spawning new processes on Windows, regenerate
             # the functions and returners.
+            # This also needed for ZeroMQ transport to reset all functions
+            # context data that could keep paretns connections. ZeroMQ will
+            # hang on polling parents connections from the child process.
             self.functions = salt.loader.minion_mods(self.opts)
             self.returners = salt.loader.returners(self.opts, self.functions)
         ret = {'id': self.opts.get('id', 'master'),
                'fun': func,
+               'fun_args': [],
                'schedule': data['name'],
                'jid': salt.utils.jid.gen_jid()}
 
@@ -595,7 +760,7 @@ class Schedule(object):
                 log.warning('schedule: The metadata parameter must be '
                             'specified as a dictionary.  Ignoring.')
 
-        salt.utils.appendproctitle(ret['jid'])
+        salt.utils.appendproctitle('{0} {1}'.format(self.__class__.__name__, ret['jid']))
 
         proc_fn = os.path.join(
             salt.minion.get_proc_dir(self.opts['cachedir']),
@@ -610,37 +775,26 @@ class Schedule(object):
         # dict we treat it like it was there and is True
         if 'jid_include' not in data or data['jid_include']:
             jobcount = 0
-            for basefilename in os.listdir(salt.minion.get_proc_dir(self.opts['cachedir'])):
-                fn_ = os.path.join(salt.minion.get_proc_dir(self.opts['cachedir']), basefilename)
-                if not os.path.exists(fn_):
-                    log.debug('schedule.handle_func: {0} was processed '
-                              'in another thread, skipping.'.format(
-                                  basefilename))
-                    continue
-                with salt.utils.fopen(fn_, 'rb') as fp_:
-                    job = salt.payload.Serial(self.opts).load(fp_)
-                    if job:
-                        if 'schedule' in job:
-                            log.debug('schedule.handle_func: Checking job against '
-                                      'fun {0}: {1}'.format(ret['fun'], job))
-                            if ret['schedule'] == job['schedule'] and os_is_running(job['pid']):
-                                jobcount += 1
-                                log.debug(
-                                    'schedule.handle_func: Incrementing jobcount, now '
-                                    '{0}, maxrunning is {1}'.format(
-                                        jobcount, data['maxrunning']))
-                                if jobcount >= data['maxrunning']:
-                                    log.debug(
-                                        'schedule.handle_func: The scheduled job {0} '
-                                        'was not started, {1} already running'.format(
-                                            ret['schedule'], data['maxrunning']))
-                                    return False
-                    else:
-                        try:
-                            log.info('Invalid job file found.  Removing.')
-                            os.remove(fn_)
-                        except OSError:
-                            log.info('Unable to remove file: {0}.'.format(fn_))
+            for job in salt.utils.minion.running(self.opts):
+                if 'schedule' in job:
+                    log.debug('schedule.handle_func: Checking job against '
+                              'fun {0}: {1}'.format(ret['fun'], job))
+                    if ret['schedule'] == job['schedule'] and os_is_running(job['pid']):
+                        jobcount += 1
+                        log.debug(
+                            'schedule.handle_func: Incrementing jobcount, now '
+                            '{0}, maxrunning is {1}'.format(
+                                jobcount, data['maxrunning']))
+                        if jobcount >= data['maxrunning']:
+                            log.debug(
+                                'schedule.handle_func: The scheduled job {0} '
+                                'was not started, {1} already running'.format(
+                                    ret['schedule'], data['maxrunning']))
+                            return False
+
+        if multiprocessing_enabled and not salt.utils.is_windows():
+            # Reconfigure multiprocessing logging after daemonizing
+            log_setup.setup_multiprocessing_logging()
 
         # Don't *BEFORE* to go into try to don't let it triple execute the finally section.
         salt.utils.daemonize_if(self.opts)
@@ -659,10 +813,12 @@ class Schedule(object):
             args = tuple()
             if 'args' in data:
                 args = data['args']
+                ret['fun_args'].extend(data['args'])
 
             kwargs = {}
             if 'kwargs' in data:
                 kwargs = data['kwargs']
+                ret['fun_args'].append(data['kwargs'])
 
             if func not in self.functions:
                 ret['return'] = self.functions.missing_fun_string(func)
@@ -681,8 +837,10 @@ class Schedule(object):
 
             data_returner = data.get('returner', None)
             if data_returner or self.schedule_returner:
-                if 'returner_config' in data:
-                    ret['ret_config'] = data['returner_config']
+                if 'return_config' in data:
+                    ret['ret_config'] = data['return_config']
+                if 'return_kwargs' in data:
+                    ret['ret_kwargs'] = data['return_kwargs']
                 rets = []
                 for returner in [data_returner, self.schedule_returner]:
                     if isinstance(returner, str):
@@ -727,11 +885,14 @@ class Schedule(object):
                         # Send back to master so the job is included in the job list
                         mret = ret.copy()
                         mret['jid'] = 'req'
-                        channel = salt.transport.Channel.factory(self.opts, usage='salt_schedule')
+                        if data.get('return_job') == 'nocache':
+                            # overwrite 'req' to signal to master that this job shouldn't be stored
+                            mret['jid'] = 'nocache'
+                        event = salt.utils.event.get_event('minion', opts=self.opts, listen=False)
                         load = {'cmd': '_return', 'id': self.opts['id']}
                         for key, value in six.iteritems(mret):
                             load[key] = value
-                        channel.send(load)
+                        event.fire_event(load, '__schedule_return')
 
                 log.debug('schedule.handle_func: Removing {0}'.format(proc_fn))
                 os.unlink(proc_fn)
@@ -745,6 +906,10 @@ class Schedule(object):
                     # Otherwise, failing to delete this file is not something
                     # we can cleanly handle.
                     raise
+            finally:
+                if multiprocessing_enabled:
+                    # Let's make sure we exit the process!
+                    sys.exit(salt.defaults.exitcodes.EX_GENERIC)
 
     def eval(self):
         '''
@@ -788,7 +953,7 @@ class Schedule(object):
 
             if 'until' in data:
                 if not _WHEN_SUPPORTED:
-                    log.error('Missing python-dateutil.'
+                    log.error('Missing python-dateutil. '
                               'Ignoring until.')
                 else:
                     until__ = dateutil_parser.parse(data['until'])
@@ -801,7 +966,7 @@ class Schedule(object):
 
             if 'after' in data:
                 if not _WHEN_SUPPORTED:
-                    log.error('Missing python-dateutil.'
+                    log.error('Missing python-dateutil. '
                               'Ignoring after.')
                 else:
                     after__ = dateutil_parser.parse(data['after'])
@@ -823,7 +988,7 @@ class Schedule(object):
 
             if any(i <= schedule_keys for i in invalid_sched_combos):
                 log.error('Unable to use "{0}" options together. Ignoring.'
-                        .format('", "'.join(scheduling_elements)))
+                          .format('", "'.join(scheduling_elements)))
                 continue
 
             invalid_time_combos = []
@@ -834,8 +999,8 @@ class Schedule(object):
 
             if any(set(x) <= schedule_keys for x in invalid_time_combos):
                 log.error('Unable to use "{0}" with "{1}" options. Ignoring'
-                        .format('", "'.join(time_elements),
-                            '", "'.join(scheduling_elements)))
+                          .format('", "'.join(time_elements),
+                                  '", "'.join(scheduling_elements)))
                 continue
 
             if True in [True for item in time_elements if item in data]:
@@ -862,8 +1027,8 @@ class Schedule(object):
 
             elif 'when' in data:
                 if not _WHEN_SUPPORTED:
-                    log.error('Missing python-dateutil.'
-                              'Ignoring job {0}'.format(job))
+                    log.error('Missing python-dateutil. '
+                              'Ignoring job {0}.'.format(job))
                     continue
 
                 if isinstance(data['when'], list):
@@ -873,7 +1038,7 @@ class Schedule(object):
                                 i in self.opts['pillar']['whens']):
                             if not isinstance(self.opts['pillar']['whens'],
                                               dict):
-                                log.error('Pillar item "whens" must be dict.'
+                                log.error('Pillar item "whens" must be dict. '
                                           'Ignoring')
                                 continue
                             __when = self.opts['pillar']['whens'][i]
@@ -899,7 +1064,7 @@ class Schedule(object):
                             try:
                                 when__ = dateutil_parser.parse(i)
                             except ValueError:
-                                log.error('Invalid date string {0}.'
+                                log.error('Invalid date string {0}. '
                                           'Ignoring job {1}.'.format(i, job))
                                 continue
                         when = int(time.mktime(when__.timetuple()))
@@ -918,8 +1083,8 @@ class Schedule(object):
                             data['_when'] = when
                         seconds = when - now
 
-                        # scheduled time is in the past
-                        if seconds < 0:
+                        # scheduled time is in the past and the run was not triggered before
+                        if seconds < 0 and not data.get('_when_run', False):
                             continue
 
                         if '_when_run' not in data:
@@ -971,8 +1136,8 @@ class Schedule(object):
                     now = int(time.time())
                     seconds = when - now
 
-                    # scheduled time is in the past
-                    if seconds < 0:
+                    # scheduled time is in the past and the run was not triggered before
+                    if seconds < 0 and not data.get('_when_run', False):
                         continue
 
                     if '_when_run' not in data:
@@ -1028,26 +1193,17 @@ class Schedule(object):
                         else:
                             data['_seconds'] = 0
 
-            if job in self.intervals:
-                if 'when' in data:
-                    if seconds == 0:
-                        if data['_when_run']:
-                            data['_when_run'] = False
-                            run = True
-                elif 'cron' in data:
-                    if seconds == 1:
-                        run = True
-                else:
-                    if now - self.intervals[job] >= seconds:
-                        run = True
+            if 'when' in data:
+                # scheduled time is now or in the past, and the run was triggered before
+                if seconds <= 0 and data['_when_run']:
+                    data['_when_run'] = False
+                    run = True
+            elif 'cron' in data:
+                if seconds == 1:
+                    run = True
             else:
-                if 'when' in data:
-                    if seconds == 0:
-                        if data['_when_run']:
-                            data['_when_run'] = False
-                            run = True
-                elif 'cron' in data:
-                    if seconds == 1:
+                if job in self.intervals:
+                    if now - self.intervals[job] >= seconds:
                         run = True
                 else:
                     # If run_on_start is True, the job will run when the Salt
@@ -1137,6 +1293,8 @@ class Schedule(object):
                              'job {0}, defaulting to 1.'.format(job))
                     data['maxrunning'] = 1
 
+            multiprocessing_enabled = self.opts.get('multiprocessing', True)
+
             if salt.utils.is_windows():
                 # Temporarily stash our function references.
                 # You can't pickle function references, and pickling is
@@ -1146,13 +1304,21 @@ class Schedule(object):
                 returners = self.returners
                 self.returners = {}
             try:
-                if self.opts.get('multiprocessing', True):
-                    thread_cls = multiprocessing.Process
+                if multiprocessing_enabled:
+                    thread_cls = SignalHandlingMultiprocessingProcess
                 else:
                     thread_cls = threading.Thread
-                proc = thread_cls(target=self.handle_func, args=(func, data))
-                proc.start()
-                if self.opts.get('multiprocessing', True):
+                proc = thread_cls(target=self.handle_func, args=(multiprocessing_enabled, func, data))
+
+                if multiprocessing_enabled:
+                    with default_signals(signal.SIGINT, signal.SIGTERM):
+                        # Reset current signals before starting the process in
+                        # order not to inherit the current signal handlers
+                        proc.start()
+                else:
+                    proc.start()
+
+                if multiprocessing_enabled:
                     proc.join()
             finally:
                 self.intervals[job] = now
