@@ -14,6 +14,7 @@ ensure_in_syspath('../../')
 
 # Import salt libs
 import integration
+import salt.utils
 from salt.modules import mysql as mysqlmod
 
 # Import 3rd-party libs
@@ -28,6 +29,9 @@ try:
 except Exception:
     NO_MYSQL = True
 
+if not salt.utils.which('mysqladmin'):
+    NO_MYSQL = True
+
 
 @skipIf(
     NO_MYSQL,
