@@ -7,17 +7,16 @@ Tests for the file state
 # Import python libs
 from __future__ import absolute_import
 from distutils.version import LooseVersion
+import errno
 import glob
-import grp
 import os
-import pwd
+import re
 import sys
 import shutil
 import stat
 import tempfile
 import textwrap
 import filecmp
-import textwrap
 
 # Import 3rd-party libs
 from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin
@@ -36,6 +35,31 @@ ensure_in_syspath('../../')
 import integration
 import salt.utils
 
+HAS_PWD = True
+try:
+    import pwd
+except ImportError:
+    HAS_PWD = False
+
+HAS_GRP = True
+try:
+    import grp
+except ImportError:
+    HAS_GRP = False
+
+IS_ADMIN = False
+IS_WINDOWS = False
+if salt.utils.is_windows():
+    IS_WINDOWS = True
+    import salt.utils.win_functions
+    current_user = salt.utils.win_functions.get_current_user()
+    if current_user == 'SYSTEM':
+        IS_ADMIN = True
+    else:
+        IS_ADMIN = salt.utils.win_functions.is_admin(current_user)
+else:
+    IS_ADMIN = os.geteuid() == 0
+
 # Import 3rd-party libs
 import salt.ext.six as six
 
@@ -50,9 +74,80 @@ except ImportError:
     HAS_GIT_PYTHON = False
 
 STATE_DIR = os.path.join(integration.FILES, 'file', 'base')
-FILEPILLAR = '/tmp/filepillar-python'
-FILEPILLARDEF = '/tmp/filepillar-defaultvalue'
-FILEPILLARGIT = '/tmp/filepillar-bar'
+if IS_WINDOWS:
+    FILEPILLAR = 'C:\\Windows\\Temp\\filepillar-python'
+    FILEPILLARDEF = 'C:\\Windows\\Temp\\filepillar-defaultvalue'
+    FILEPILLARGIT = 'C:\\Windows\\Temp\\filepillar-bar'
+else:
+    FILEPILLAR = '/tmp/filepillar-python'
+    FILEPILLARDEF = '/tmp/filepillar-defaultvalue'
+    FILEPILLARGIT = '/tmp/filepillar-bar'
+
+
+def _test_managed_file_mode_keep_helper(testcase, local=False):
+    '''
+    DRY helper function to run the same test with a local or remote path
+    '''
+    rel_path = 'grail/scene33'
+    name = os.path.join(integration.TMP, os.path.basename(rel_path))
+    grail_fs_path = os.path.join(integration.FILES, 'file', 'base', rel_path)
+    grail = 'salt://' + rel_path if not local else grail_fs_path
+
+    # Get the current mode so that we can put the file back the way we
+    # found it when we're done.
+    grail_fs_mode = os.stat(grail_fs_path).st_mode
+    initial_mode = 504    # 0770 octal
+    new_mode_1 = 384      # 0600 octal
+    new_mode_2 = 420      # 0644 octal
+
+    # Set the initial mode, so we can be assured that when we set the mode
+    # to "keep", we're actually changing the permissions of the file to the
+    # new mode.
+    ret = testcase.run_state(
+        'file.managed',
+        name=name,
+        mode=oct(initial_mode),
+        source=grail,
+    )
+
+    if IS_WINDOWS:
+        testcase.assertSaltFalseReturn(ret)
+        return
+
+    testcase.assertSaltTrueReturn(ret)
+
+    try:
+        # Update the mode on the fileserver (pass 1)
+        os.chmod(grail_fs_path, new_mode_1)
+        ret = testcase.run_state(
+            'file.managed',
+            name=name,
+            mode='keep',
+            source=grail,
+        )
+        testcase.assertSaltTrueReturn(ret)
+        managed_mode = stat.S_IMODE(os.stat(name).st_mode)
+        testcase.assertEqual(oct(managed_mode), oct(new_mode_1))
+        # Update the mode on the fileserver (pass 2)
+        # This assures us that if the file in file_roots was originally set
+        # to the same mode as new_mode_1, we definitely get an updated mode
+        # this time.
+        os.chmod(grail_fs_path, new_mode_2)
+        ret = testcase.run_state(
+            'file.managed',
+            name=name,
+            mode='keep',
+            source=grail,
+        )
+        testcase.assertSaltTrueReturn(ret)
+        managed_mode = stat.S_IMODE(os.stat(name).st_mode)
+        testcase.assertEqual(oct(managed_mode), oct(new_mode_2))
+    except Exception:
+        raise
+    finally:
+        # Set the mode of the file in the file_roots back to what it
+        # originally was.
+        os.chmod(grail_fs_path, grail_fs_mode)
 
 
 class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
@@ -66,6 +161,15 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         '''
         name = os.path.join(integration.TMP, 'symlink')
         tgt = os.path.join(integration.TMP, 'target')
+
+        # Windows must have a source directory to link to
+        if salt.utils.is_windows() and not os.path.isdir(tgt):
+            os.mkdir(tgt)
+
+        # Windows cannot create a symlink if it already exists
+        if salt.utils.is_windows() and self.run_function('file.is_link', [name]):
+            self.run_function('file.remove', [name])
+
         ret = self.run_state('file.symlink', name=name, target=tgt)
         self.assertSaltTrueReturn(ret)
 
@@ -106,15 +210,23 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         file.absent
         '''
         name = os.path.join(integration.TMP, 'link_to_kill')
-        if not os.path.islink('{0}.tgt'.format(name)):
-            os.symlink(name, '{0}.tgt'.format(name))
+        tgt = '{0}.tgt'.format(name)
+
+        # Windows must have a source directory to link to
+        if salt.utils.is_windows() and not os.path.isdir(tgt):
+            os.mkdir(tgt)
+
+        if not self.run_function('file.is_link', [name]):
+            self.run_function('file.symlink', [tgt, name])
+
         ret = self.run_state('file.absent', name=name)
+
         try:
             self.assertSaltTrueReturn(ret)
-            self.assertFalse(os.path.islink(name))
+            self.assertFalse(self.run_function('file.is_link', [name]))
         finally:
-            if os.path.islink('{0}.tgt'.format(name)):
-                os.unlink('{0}.tgt'.format(name))
+            if self.run_function('file.is_link', [name]):
+                self.run_function('file.remove', [name])
 
     def test_test_absent(self):
         '''
@@ -158,12 +270,31 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
             'file.managed', name=name, mode='0770', source='salt://grail/scene33'
         )
 
+        if IS_WINDOWS:
+            expected = 'The \'mode\' option is not supported on Windows'
+            self.assertEqual(ret[ret.keys()[0]]['comment'], expected)
+            self.assertSaltFalseReturn(ret)
+            return
+
         resulting_mode = stat.S_IMODE(
             os.stat(name).st_mode
         )
         self.assertEqual(oct(desired_mode), oct(resulting_mode))
         self.assertSaltTrueReturn(ret)
 
+    def test_managed_file_mode_keep(self):
+        '''
+        Test using "mode: keep" in a file.managed state
+        '''
+        _test_managed_file_mode_keep_helper(self, local=False)
+
+    def test_managed_file_mode_keep_local_source(self):
+        '''
+        Test using "mode: keep" in a file.managed state, with a local file path
+        as the source.
+        '''
+        _test_managed_file_mode_keep_helper(self, local=True)
+
     def test_managed_file_mode_file_exists_replace(self):
         '''
         file.managed, existing file with replace=True, change permissions
@@ -175,6 +306,12 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
             'file.managed', name=name, mode=oct(initial_mode), source='salt://grail/scene33'
         )
 
+        if IS_WINDOWS:
+            expected = 'The \'mode\' option is not supported on Windows'
+            self.assertEqual(ret[ret.keys()[0]]['comment'], expected)
+            self.assertSaltFalseReturn(ret)
+            return
+
         resulting_mode = stat.S_IMODE(
             os.stat(name).st_mode
         )
@@ -201,6 +338,12 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
             'file.managed', name=name, replace=True, mode=oct(initial_mode), source='salt://grail/scene33'
         )
 
+        if IS_WINDOWS:
+            expected = 'The \'mode\' option is not supported on Windows'
+            self.assertEqual(ret[ret.keys()[0]]['comment'], expected)
+            self.assertSaltFalseReturn(ret)
+            return
+
         ret = self.run_state(
             'file.managed', name=name, replace=False, mode=oct(desired_mode), source='salt://grail/scene33'
         )
@@ -210,6 +353,24 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         self.assertEqual(oct(desired_mode), oct(resulting_mode))
         self.assertSaltTrueReturn(ret)
 
+    @destructiveTest
+    def test_managed_file_with_grains_data(self):
+        '''
+        Test to ensure we can render grains data into a managed
+        file.
+        '''
+        grain_path = os.path.join(integration.TMP, 'file-grain-test')
+        self.run_function('grains.set', ['grain_path', grain_path])
+        state_file = 'file-grainget'
+
+        self.run_function('state.sls', [state_file])
+        self.assertTrue(os.path.exists(grain_path))
+
+        with salt.utils.fopen(grain_path, 'r') as fp_:
+            file_contents = fp_.readlines()
+
+        self.assertTrue(re.match('^minion$', file_contents[0]))
+
     @destructiveTest
     def test_managed_file_with_pillar_sls(self):
         '''
@@ -217,6 +378,7 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         is rendered properly and file is created.
         '''
         state_name = 'file-pillarget'
+
         ret = self.run_function('state.sls', [state_name])
         self.assertSaltTrueReturn(ret)
 
@@ -232,6 +394,7 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         value.
         '''
         state_name = 'file-pillardefaultget'
+
         ret = self.run_function('state.sls', [state_name])
         self.assertSaltTrueReturn(ret)
 
@@ -247,6 +410,7 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         file is rendered properly and is created.
         '''
         state_name = 'file-pillargit'
+
         ret = self.run_function('state.sls', [state_name])
         self.assertSaltTrueReturn(ret)
 
@@ -254,7 +418,7 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         check_file = self.run_function('file.file_exists', [FILEPILLARGIT])
         self.assertTrue(check_file)
 
-    @skipIf(os.geteuid() != 0, 'you must be root to run this test')
+    @skipIf(not IS_ADMIN, 'you must be root to run this test')
     def test_managed_dir_mode(self):
         '''
         Tests to ensure that file.managed creates directories with the
@@ -272,6 +436,12 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
             user=desired_owner,
             dir_mode=oct(desired_mode)  # 0777
         )
+        if IS_WINDOWS:
+            expected = 'The \'mode\' option is not supported on Windows'
+            self.assertEqual(ret[ret.keys()[0]]['comment'], expected)
+            self.assertSaltFalseReturn(ret)
+            return
+
         resulting_mode = stat.S_IMODE(
             os.stat(os.path.join(integration.TMP, 'a')).st_mode
         )
@@ -291,22 +461,23 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         self.assertSaltNoneReturn(ret)
         self.assertFalse(os.path.isfile(name))
 
-    def test_managed_show_diff_false(self):
+    def test_managed_show_changes_false(self):
         '''
         file.managed test interface
         '''
         name = os.path.join(integration.TMP, 'grail_not_scene33')
         with salt.utils.fopen(name, 'wb') as fp_:
-            fp_.write('test_managed_show_diff_false\n')
+            fp_.write('test_managed_show_changes_false\n')
 
         ret = self.run_state(
             'file.managed', name=name, source='salt://grail/scene33',
-            show_diff=False
+            show_changes=False
         )
 
         changes = next(six.itervalues(ret))['changes']
-        self.assertEqual('<show_diff=False>', changes['diff'])
+        self.assertEqual('<show_changes=False>', changes['diff'])
 
+    @skipIf(IS_WINDOWS, 'Don\'t know how to fix for Windows')
     def test_managed_escaped_file_path(self):
         '''
         file.managed test that 'salt://|' protects unusual characters in file path
@@ -352,44 +523,52 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         managed_files = {}
         state_keys = {}
         for typ in ('bool', 'str', 'int', 'float', 'list', 'dict'):
-            managed_files[typ] = tempfile.mkstemp()[1]
+            fd_, managed_files[typ] = tempfile.mkstemp()
+
+            # Release the handle so they can be removed in Windows
+            try:
+                os.close(fd_)
+            except OSError as exc:
+                if exc.errno != errno.EBADF:
+                    raise exc
+
             state_keys[typ] = 'file_|-{0} file_|-{1}_|-managed'.format(typ, managed_files[typ])
         try:
-            with salt.utils.fopen(state_file, 'w') as fp_:
-                fp_.write(textwrap.dedent('''\
-                bool file:
-                  file.managed:
-                    - name: {bool}
-                    - contents: True
+            with salt.utils.fopen(state_file, 'w') as fd_:
+                fd_.write(textwrap.dedent('''\
+                    bool file:
+                      file.managed:
+                        - name: {bool}
+                        - contents: True
 
-                str file:
-                  file.managed:
-                    - name: {str}
-                    - contents: Salt was here.
+                    str file:
+                      file.managed:
+                        - name: {str}
+                        - contents: Salt was here.
 
-                int file:
-                  file.managed:
-                    - name: {int}
-                    - contents: 340282366920938463463374607431768211456
+                    int file:
+                      file.managed:
+                        - name: {int}
+                        - contents: 340282366920938463463374607431768211456
 
-                float file:
-                  file.managed:
-                    - name: {float}
-                    - contents: 1.7518e-45  # gravitational coupling constant
+                    float file:
+                      file.managed:
+                        - name: {float}
+                        - contents: 1.7518e-45  # gravitational coupling constant
 
-                list file:
-                  file.managed:
-                    - name: {list}
-                    - contents: [1, 1, 2, 3, 5, 8, 13]
+                    list file:
+                      file.managed:
+                        - name: {list}
+                        - contents: [1, 1, 2, 3, 5, 8, 13]
 
-                dict file:
-                  file.managed:
-                    - name: {dict}
-                    - contents:
-                        C: charge
-                        P: parity
-                        T: time
-                '''.format(**managed_files)))
+                    dict file:
+                      file.managed:
+                        - name: {dict}
+                        - contents:
+                            C: charge
+                            P: parity
+                            T: time
+                    '''.format(**managed_files)))
 
             ret = self.run_function('state.sls', [state_name])
             for typ in state_keys:
@@ -409,6 +588,48 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         self.assertSaltTrueReturn(ret)
         self.assertTrue(os.path.isdir(name))
 
+    @skipIf(IS_WINDOWS, 'Mode not available in Windows')
+    def test_directory_max_depth(self):
+        '''
+        file.directory
+        Test the max_depth option by iteratively increasing the depth and
+        checking that no changes deeper than max_depth have been attempted
+        '''
+
+        def _get_oct_mode(name):
+            '''
+            Return a string octal representation of the permissions for name
+            '''
+            return oct(os.stat(name).st_mode & 0o777)
+
+        top = os.path.join(integration.TMP, 'top_dir')
+        sub = os.path.join(top, 'sub_dir')
+        subsub = os.path.join(sub, 'sub_sub_dir')
+        dirs = [top, sub, subsub]
+
+        initial_mode = '0111'
+        changed_mode = '0555'
+
+        if not os.path.isdir(subsub):
+            os.makedirs(subsub, int(initial_mode, 8))
+
+        try:
+            for depth in range(0, 3):
+                ret = self.run_state('file.directory',
+                                     name=top,
+                                     max_depth=depth,
+                                     dir_mode=changed_mode,
+                                     recurse=['mode'])
+                self.assertSaltTrueReturn(ret)
+                for changed_dir in dirs[0:depth+1]:
+                    self.assertEqual(changed_mode,
+                                     _get_oct_mode(changed_dir))
+                for untouched_dir in dirs[depth+1:]:
+                    self.assertEqual(initial_mode,
+                                     _get_oct_mode(untouched_dir))
+        finally:
+            shutil.rmtree(top)
+
     def test_test_directory(self):
         '''
         file.directory
@@ -470,10 +691,14 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         with salt.utils.fopen(keepfile, 'w'):
             pass
 
+        exclude_pat = 'E@^straydir(|/keepfile)$'
+        if salt.utils.is_windows():
+            exclude_pat = 'E@^straydir(|\\\\keepfile)$'
+
         ret = self.run_state('file.directory',
                              name=name,
                              clean=True,
-                             exclude_pat='E@^straydir(|/keepfile)$')
+                             exclude_pat=exclude_pat)
 
         try:
             self.assertSaltTrueReturn(ret)
@@ -485,7 +710,7 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
 
     def test_test_directory_clean_exclude(self):
         '''
-        file.directory test with clean=True and exclude_pat set
+        file.directory with test=True, clean=True and exclude_pat set
         '''
         name = os.path.join(integration.TMP, 'directory_clean_dir')
         if not os.path.isdir(name):
@@ -507,13 +732,18 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         with salt.utils.fopen(keepfile, 'w'):
             pass
 
+        exclude_pat = 'E@^straydir(|/keepfile)$'
+        if salt.utils.is_windows():
+            exclude_pat = 'E@^straydir(|\\\\keepfile)$'
+
         ret = self.run_state('file.directory',
                              test=True,
                              name=name,
                              clean=True,
-                             exclude_pat='E@^straydir(|/keepfile)$')
+                             exclude_pat=exclude_pat)
 
         comment = next(six.itervalues(ret))['comment']
+
         try:
             self.assertSaltNoneReturn(ret)
             self.assertTrue(os.path.exists(strayfile))
@@ -675,6 +905,32 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
             if os.path.isdir(name):
                 shutil.rmtree(name, ignore_errors=True)
 
+    def test_recurse_specific_env_in_url(self):
+        '''
+        file.recurse passing __env__
+        '''
+        name = os.path.join(integration.TMP, 'recurse_dir_prod_env')
+        ret = self.run_state('file.recurse',
+                             name=name,
+                             source='salt://holy?saltenv=prod')
+        try:
+            self.assertSaltTrueReturn(ret)
+            self.assertTrue(os.path.isfile(os.path.join(name, '32', 'scene')))
+        finally:
+            if os.path.isdir(name):
+                shutil.rmtree(name, ignore_errors=True)
+
+        name = os.path.join(integration.TMP, 'recurse_dir_prod_env')
+        ret = self.run_state('file.recurse',
+                             name=name,
+                             source='salt://holy?saltenv=prod')
+        try:
+            self.assertSaltTrueReturn(ret)
+            self.assertTrue(os.path.isfile(os.path.join(name, '32', 'scene')))
+        finally:
+            if os.path.isdir(name):
+                shutil.rmtree(name, ignore_errors=True)
+
     def test_test_recurse(self):
         '''
         file.recurse test interface
@@ -881,7 +1137,7 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
 
         # Case description:
 
-        The tested multfile contains multiple lines not matching the pattern or replacement in any way
+        The tested multifile contains multiple lines not matching the pattern or replacement in any way
         The replacement pattern should be prepended to the file
         '''
         test_name = 'test_replace_issue_18612_prepend'
@@ -924,7 +1180,7 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
 
         # Case description:
 
-        The tested multfile contains multiple lines not matching the pattern or replacement in any way
+        The tested multifile contains multiple lines not matching the pattern or replacement in any way
         The replacement pattern should be appended to the file
         '''
         test_name = 'test_replace_issue_18612_append'
@@ -967,7 +1223,7 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
 
         # Case description:
 
-        The tested multfile contains multiple lines not matching the pattern or replacement in any way
+        The tested multifile contains multiple lines not matching the pattern or replacement in any way
         The 'not_found_content' value should be appended to the file
         '''
         test_name = 'test_replace_issue_18612_append_not_found_content'
@@ -1207,6 +1463,11 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
             # write a line to file
             with salt.utils.fopen(name, 'w+') as fp_:
                 fp_.write('comment_me')
+
+            # Look for changes with test=True: return should be "None" at the first run
+            ret = self.run_state('file.comment', test=True, name=name, regex='^comment')
+            self.assertSaltNoneReturn(ret)
+
             # comment once
             ret = self.run_state('file.comment', name=name, regex='^comment')
             # result is positive
@@ -1217,11 +1478,17 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
 
             # comment twice
             ret = self.run_state('file.comment', name=name, regex='^comment')
+
             # result is still positive
             self.assertSaltTrueReturn(ret)
             # line is still commented
             with salt.utils.fopen(name, 'r') as fp_:
                 self.assertTrue(fp_.read().startswith('#comment'))
+
+            # Test previously commented file returns "True" now and not "None" with test=True
+            ret = self.run_state('file.comment', test=True, name=name, regex='^comment')
+            self.assertSaltTrueReturn(ret)
+
         finally:
             os.remove(name)
 
@@ -1581,6 +1848,7 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
             if os.path.isfile(tmp_file):
                 os.remove(tmp_file)
 
+    @skipIf(IS_WINDOWS, 'Mode not available in Windows')
     def test_issue_2726_mode_kwarg(self):
         testcase_temp_dir = os.path.join(integration.TMP, 'issue_2726')
         # Let's test for the wrong usage approach
@@ -1595,7 +1863,7 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         ]
         try:
             ret = self.run_function(
-                'state.template_str', ['\n'.join(bad_template)]
+                'state.template_str', [os.linesep.join(bad_template)]
             )
             self.assertSaltFalseReturn(ret)
             self.assertInSaltComment(
@@ -1625,7 +1893,7 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         ]
         try:
             ret = self.run_function(
-                'state.template_str', ['\n'.join(good_template)]
+                'state.template_str', [os.linesep.join(good_template)]
             )
             self.assertSaltTrueReturn(ret)
         finally:
@@ -1678,14 +1946,15 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
             '    - show_changes: True',
             '']
         with salt.utils.fopen(template_path, 'w') as fp_:
-            fp_.write('\n'.join(sls_template).format(testcase_filedest))
+            fp_.write(
+                os.linesep.join(sls_template).format(testcase_filedest))
 
         try:
             ret = self.run_function('state.sls', mods='issue-8343')
             for name, step in six.iteritems(ret):
                 self.assertSaltTrueReturn({name: step})
             with salt.utils.fopen(testcase_filedest) as fp_:
-                contents = fp_.read().split('\n')
+                contents = fp_.read().split(os.linesep)
             self.assertEqual(
                 ['#-- start salt managed zonestart -- PLEASE, DO NOT EDIT',
                  'foo',
@@ -1704,6 +1973,7 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
                 os.unlink(filename)
 
     def test_issue_11003_immutable_lazy_proxy_sum(self):
+        # causes the Import-Module ServerManager error on Windows
         template_path = os.path.join(integration.TMP_STATE_TREE, 'issue-11003.sls')
         testcase_filedest = os.path.join(integration.TMP, 'issue-11003.txt')
         sls_template = [
@@ -1743,14 +2013,14 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         ]
 
         with salt.utils.fopen(template_path, 'w') as fp_:
-            fp_.write('\n'.join(sls_template).format(testcase_filedest))
+            fp_.write(os.linesep.join(sls_template).format(testcase_filedest))
 
         try:
             ret = self.run_function('state.sls', mods='issue-11003')
             for name, step in six.iteritems(ret):
                 self.assertSaltTrueReturn({name: step})
             with salt.utils.fopen(testcase_filedest) as fp_:
-                contents = fp_.read().split('\n')
+                contents = fp_.read().split(os.linesep)
             self.assertEqual(
                 ['#',
                  '#-- start managed zone PLEASE, DO NOT EDIT',
@@ -1767,9 +2037,10 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
             for filename in glob.glob('{0}.bak*'.format(testcase_filedest)):
                 os.unlink(filename)
 
+    @skipIf(IS_WINDOWS, 'Don\'t know how to fix for Windows')
     def test_issue_8947_utf8_sls(self):
         '''
-        Test some file operation with utf-8 chararacters on the sls
+        Test some file operation with utf-8 characters on the sls
 
         This is more generic than just a file test. Feel free to move
         '''
@@ -1832,7 +2103,7 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
             '      - cmd: some-utf8-file-content-remove',
         ]
         with salt.utils.fopen(template_path, 'w') as fp_:
-            fp_.write('\n'.join(template_lines))
+            fp_.write(os.linesep.join(template_lines))
         try:
             ret = self.run_function('state.sls', mods='issue-8947')
             if not isinstance(ret, dict):
@@ -1924,7 +2195,9 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
                 os.unlink(template_path)
 
     @destructiveTest
-    @skipIf(os.geteuid() != 0, 'you must be root to run this test')
+    @skipIf(not IS_ADMIN, 'you must be root to run this test')
+    @skipIf(not HAS_PWD, "pwd not available. Skipping test")
+    @skipIf(not HAS_GRP, "grp not available. Skipping test")
     @with_system_user_and_group('user12209', 'group12209',
                                 on_existing='delete', delete=True)
     def test_issue_12209_follow_symlinks(self, user, group):
@@ -1970,7 +2243,9 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
                 shutil.rmtree(tmp_dir)
 
     @destructiveTest
-    @skipIf(os.geteuid() != 0, 'you must be root to run this test')
+    @skipIf(not IS_ADMIN, 'you must be root to run this test')
+    @skipIf(not HAS_PWD, "pwd not available. Skipping test")
+    @skipIf(not HAS_GRP, "grp not available. Skipping test")
     @with_system_user_and_group('user12209', 'group12209',
                                 on_existing='delete', delete=True)
     def test_issue_12209_no_follow_symlinks(self, user, group):
@@ -2018,8 +2293,20 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         Test a file.managed state with a local file as the source. Test both
         with the file:// protocol designation prepended, and without it.
         '''
-        source = tempfile.mkstemp()[-1]
-        dest = tempfile.mkstemp()[-1]
+        fd_, source = tempfile.mkstemp()
+        try:
+            os.close(fd_)
+        except OSError as exc:
+            if exc.errno != errno.EBADF:
+                raise exc
+
+        fd_, dest = tempfile.mkstemp()
+        try:
+            os.close(fd_)
+        except OSError as exc:
+            if exc.errno != errno.EBADF:
+                raise exc
+
         with salt.utils.fopen(source, 'w') as fp_:
             fp_.write('{{ foo }}\n')
 
@@ -2042,7 +2329,13 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         Test the case where a source file is in the minion's local filesystem,
         and the source path is the same as the destination path.
         '''
-        source = tempfile.mkstemp()[-1]
+        fd_, source = tempfile.mkstemp()
+        try:
+            os.close(fd_)
+        except OSError as exc:
+            if exc.errno != errno.EBADF:
+                raise exc
+
         with salt.utils.fopen(source, 'w') as fp_:
             fp_.write('{{ foo }}\n')
 
@@ -2084,7 +2377,9 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         This tests for any regressions for this issue:
         https://github.com/saltstack/salt/issues/30934
         '''
-        ret = self.run_function('state.sls', mods='file_contents_pillar')
+        state_file = 'file_contents_pillar'
+
+        ret = self.run_function('state.sls', mods=state_file)
         self.assertSaltTrueReturn(ret)
 
     def tearDown(self):
