@@ -8,6 +8,8 @@
 
 # Import Python libs
 from __future__ import absolute_import
+from distutils.version import LooseVersion
+import mock
 
 try:
     import libcloud.security
@@ -15,9 +17,6 @@ try:
 except ImportError:
     HAS_LIBCLOUD = False
 
-import platform
-import os
-
 # Import Salt Libs
 from salt.cloud.clouds import dimensiondata
 from salt.exceptions import SaltCloudSystemExit
@@ -45,22 +44,12 @@ dimensiondata.__opts__ = {
 }
 VM_NAME = 'winterfell'
 
-HAS_CERTS = True
-ON_SUSE = True if 'SuSE' in platform.dist() else False
-ON_MAC = True if 'Darwin' in platform.system() else False
-
-if not os.path.exists('/etc/ssl/certs/YaST-CA.pem') and ON_SUSE:
-    if os.path.isfile('/etc/ssl/ca-bundle.pem') and HAS_LIBCLOUD:
-        libcloud.security.CA_CERTS_PATH.append('/etc/ssl/ca-bundle.pem')
-    else:
-        HAS_CERTS = False
-elif ON_MAC:
-    if os.path.isfile('/opt/local/share/curl/curl-ca-bundle.crt'):
-        pass  # libcloud will already find this file
-    elif os.path.isfile('/usr/local/etc/openssl/cert.pem'):
-        pass  # libcloud will already find this file
-    else:
-        HAS_CERTS = False
+# Use certifi if installed
+try:
+    import certifi
+    libcloud.security.CA_CERTS_PATH.append(certifi.where())
+except ImportError:
+    pass
 
 
 class ExtendedTestCase(TestCase):
@@ -77,7 +66,6 @@ class ExtendedTestCase(TestCase):
             self.assertEqual(exc.message, exc_msg)
 
 
-@skipIf(not HAS_CERTS, 'Cannot find CA cert bundle')
 @skipIf(NO_MOCK, NO_MOCK_REASON)
 @patch('salt.cloud.clouds.dimensiondata.__virtual__', MagicMock(return_value='dimensiondata'))
 class DimensionDataTestCase(ExtendedTestCase):
@@ -155,22 +143,22 @@ class DimensionDataTestCase(ExtendedTestCase):
             'default'
         )
 
-    @patch('libcloud.compute.drivers.dimensiondata.DimensionDataNodeDriver.list_nodes', MagicMock(return_value=[]))
-    def test_list_nodes(self):
-        nodes = dimensiondata.list_nodes()
-        self.assertEqual(
-            nodes,
-            {}
-        )
-
-    @patch('libcloud.compute.drivers.dimensiondata.DimensionDataNodeDriver.list_locations', MagicMock(return_value=[]))
-    def test_list_locations(self):
-        locations = dimensiondata.avail_locations()
-        self.assertEqual(
-            locations,
-            {}
-        )
+    def test_import(self):
+        """
+        Test that the module picks up installed deps
+        """
+        with patch('salt.config.check_driver_dependencies', return_value=True) as p:
+            get_deps = dimensiondata.get_dependencies()
+            self.assertEqual(get_deps, True)
+            if LooseVersion(mock.__version__) >= LooseVersion('2.0.0'):
+                p.assert_called_once()
 
+    def test_provider_matches(self):
+        """
+        Test that the first configured instance of a dimensiondata driver is matched
+        """
+        p = dimensiondata.get_configured_provider()
+        self.assertNotEqual(p, None)
 
 if __name__ == '__main__':
     from integration import run_tests
