@@ -17,9 +17,18 @@ ensure_in_syspath('../../../')
 from salt.cloud.clouds import opennebula
 from salt.exceptions import SaltCloudSystemExit, SaltCloudNotFound
 
+# Import Third Party Libs
+try:
+    from lxml import etree  # pylint: disable=W0611
+    HAS_XML_LIBS = True
+except ImportError:
+    HAS_XML_LIBS = False
+
 # Global Variables
 opennebula.__active_provider_name__ = ''
 opennebula.__opts__ = {}
+opennebula.__utils__ = {}
+opennebula.__utils__['cloud.cache_node'] = MagicMock()
 VM_NAME = 'my-vm'
 
 
@@ -761,7 +770,6 @@ class OpenNebulaTestCase(TestCase):
 
     @patch('salt.cloud.clouds.opennebula._get_node',
            MagicMock(return_value={'my-vm': {'name': 'my-vm', 'id': 0}}))
-    @patch('salt.utils.cloud.cache_node', MagicMock())
     def test_show_instance_success(self):
         '''
         Tests that the node was found successfully.
@@ -1628,6 +1636,16 @@ class OpenNebulaTestCase(TestCase):
                           call='function',
                           kwargs={'vn_id': '0'})
 
+    @skipIf(not HAS_XML_LIBS, 'cannot find lxml python library')
+    def test__get_xml(self):
+        '''
+        Tests that invalid XML raises SaltCloudSystemExit.
+        '''
+        self.assertRaises(SaltCloudSystemExit,
+                          opennebula._get_xml,
+                          "[VirtualMachinePoolInfo] User couldn't be"
+                          " authenticated, aborting call.")
+
 
 if __name__ == '__main__':
     from integration import run_tests
