@@ -14,6 +14,7 @@ from copy import deepcopy
 from salttesting import TestCase, skipIf
 from salttesting.mock import MagicMock, NO_MOCK, NO_MOCK_REASON, patch
 from salttesting.helpers import ensure_in_syspath
+from salt import config
 
 ensure_in_syspath('../../../')
 
@@ -43,6 +44,14 @@ PROVIDER_CONFIG = {
   }
 }
 VM_NAME = 'test-vm'
+PROFILE = {
+  'base-gold': {
+    'provider': 'vcenter01:vmware',
+    'datastore': 'Datastore1',
+    'resourcepool': 'Resources',
+    'folder': 'vm'
+  }
+}
 
 
 class ExtendedTestCase(TestCase):
@@ -60,6 +69,7 @@ class ExtendedTestCase(TestCase):
 
 
 @skipIf(NO_MOCK, NO_MOCK_REASON)
+@skipIf(not HAS_LIBS, 'Install pyVmomi to be able to run this test.')
 @patch('salt.cloud.clouds.vmware.__virtual__', MagicMock(return_value='vmware'))
 class VMwareTestCase(ExtendedTestCase):
     '''
@@ -792,6 +802,97 @@ class VMwareTestCase(ExtendedTestCase):
                 kwargs=None,
                 call='function')
 
+    def test_no_clonefrom_just_image(self):
+        '''
+        Tests that the profile is configured correctly when deploying using an image
+        '''
+
+        profile_additions = {
+            'image': 'some-image.iso'
+        }
+
+        provider_config = deepcopy(PROVIDER_CONFIG)
+        profile = deepcopy(PROFILE)
+        profile['base-gold'].update(profile_additions)
+
+        provider_config_additions = {
+            'profiles': profile
+        }
+        provider_config['vcenter01']['vmware'].update(provider_config_additions)
+        vm_ = {'profile': profile}
+        with patch.dict(vmware.__opts__, {'providers': provider_config}, clean=True):
+            self.assertEqual(config.is_profile_configured(vmware.__opts__, 'vcenter01:vmware',
+                                                          'base-gold', vm_=vm_), True)
+
+    def test_just_clonefrom(self):
+        '''
+        Tests that the profile is configured correctly when deploying by cloning from a template
+        '''
+
+        profile_additions = {
+            'clonefrom': 'test-template',
+            'image': 'should ignore image'
+        }
+
+        provider_config = deepcopy(PROVIDER_CONFIG)
+        profile = deepcopy(PROFILE)
+        profile['base-gold'].update(profile_additions)
+
+        provider_config_additions = {
+            'profiles': profile
+        }
+        provider_config['vcenter01']['vmware'].update(provider_config_additions)
+        vm_ = {'profile': profile}
+        with patch.dict(vmware.__opts__, {'providers': provider_config}, clean=True):
+            self.assertEqual(config.is_profile_configured(vmware.__opts__, 'vcenter01:vmware',
+                                                          'base-gold', vm_=vm_), True)
+
+    @patch('salt.cloud.clouds.vmware.randint', return_value=101)
+    def test_add_new_ide_controller_helper(self, randint_mock):
+        '''
+        Tests that creating a new controller, ensuring that it will generate a controller key
+        if one is not provided
+        '''
+        controller_label = 'Some label'
+        bus_number = 1
+        spec = vmware._add_new_ide_controller_helper(controller_label, None, bus_number)
+        self.assertEqual(spec.device.key, randint_mock.return_value)
+
+        spec = vmware._add_new_ide_controller_helper(controller_label, 200, bus_number)
+        self.assertEqual(spec.device.key, 200)
+
+        self.assertEqual(spec.device.busNumber, bus_number)
+        self.assertEqual(spec.device.deviceInfo.label, controller_label)
+        self.assertEqual(spec.device.deviceInfo.summary, controller_label)
+
+    def test_manage_devices_just_cd(self):
+        '''
+        Tests that when adding IDE/CD drives, controller keys will be in the apparent
+        safe-range on ESX 5.5 but randomly generated on other versions (i.e. 6)
+        '''
+        device_map = {
+            'ide': {
+                'IDE 0': {},
+                'IDE 1': {}
+            },
+            'cd': {
+                'CD/DVD Drive 1': {'controller': 'IDE 0'}
+            }
+        }
+        with patch('salt.cloud.clouds.vmware.get_vcenter_version', return_value='VMware ESXi 5.5.0'):
+            specs = vmware._manage_devices(device_map, vm=None)['device_specs']
+
+            self.assertEqual(specs[0].device.key, vmware.SAFE_ESX_5_5_CONTROLLER_KEY_INDEX)
+            self.assertEqual(specs[1].device.key, vmware.SAFE_ESX_5_5_CONTROLLER_KEY_INDEX+1)
+            self.assertEqual(specs[2].device.controllerKey, vmware.SAFE_ESX_5_5_CONTROLLER_KEY_INDEX)
+
+        with patch('salt.cloud.clouds.vmware.get_vcenter_version', return_value='VMware ESXi 6'):
+            with patch('salt.cloud.clouds.vmware.randint', return_value=100) as first_key:
+                specs = vmware._manage_devices(device_map, vm=None)['device_specs']
+
+                self.assertEqual(specs[0].device.key, first_key.return_value)
+                self.assertEqual(specs[2].device.controllerKey, first_key.return_value)
+
     def test_add_host_no_host_in_kwargs(self):
         '''
         Tests that a SaltCloudSystemExit is raised when host is not present in
@@ -1098,6 +1199,83 @@ class VMwareTestCase(ExtendedTestCase):
             call='function')
 
 
+class CloneFromSnapshotTest(TestCase):
+    '''
+    Test functionality to clone from snapshot
+    '''
+    @skipIf(HAS_LIBS is False, "Install pyVmomi to be able to run this unit test.")
+    def test_quick_linked_clone(self):
+        '''
+        Test that disk move type is
+        set to createNewChildDiskBacking
+        '''
+        self._test_clone_type(vmware.QUICK_LINKED_CLONE)
+
+    @skipIf(HAS_LIBS is False, "Install pyVmomi to be able to run this unit test.")
+    def test_current_state_linked_clone(self):
+        '''
+        Test that disk move type is
+        set to moveChildMostDiskBacking
+        '''
+        self._test_clone_type(vmware.CURRENT_STATE_LINKED_CLONE)
+
+    @skipIf(HAS_LIBS is False, "Install pyVmomi to be able to run this unit test.")
+    def test_copy_all_disks_full_clone(self):
+        '''
+        Test that disk move type is
+        set to moveAllDiskBackingsAndAllowSharing
+        '''
+        self._test_clone_type(vmware.COPY_ALL_DISKS_FULL_CLONE)
+
+    @skipIf(HAS_LIBS is False, "Install pyVmomi to be able to run this unit test.")
+    def test_flatten_all_all_disks_full_clone(self):
+        '''
+        Test that disk move type is
+        set to moveAllDiskBackingsAndDisallowSharing
+        '''
+        self._test_clone_type(vmware.FLATTEN_DISK_FULL_CLONE)
+
+    @skipIf(HAS_LIBS is False, "Install pyVmomi to be able to run this unit test.")
+    def test_raises_error_for_invalid_disk_move_type(self):
+        '''
+        Test that invalid disk move type
+        raises error
+        '''
+        with self.assertRaises(SaltCloudSystemExit):
+            self._test_clone_type('foobar')
+
+    def _test_clone_type(self, clone_type):
+        '''
+        Assertions for checking that a certain clone type
+        works
+        '''
+        obj_ref = MagicMock()
+        obj_ref.snapshot = vim.vm.Snapshot(None, None)
+        obj_ref.snapshot.currentSnapshot = vim.vm.Snapshot(None, None)
+        clone_spec = vmware.handle_snapshot(
+            vim.vm.ConfigSpec(),
+            obj_ref,
+            vim.vm.RelocateSpec(),
+            False,
+            {'snapshot': {
+                'disk_move_type': clone_type}})
+        self.assertEqual(clone_spec.location.diskMoveType, clone_type)
+
+        obj_ref2 = MagicMock()
+        obj_ref2.snapshot = vim.vm.Snapshot(None, None)
+        obj_ref2.snapshot.currentSnapshot = vim.vm.Snapshot(None, None)
+
+        clone_spec2 = vmware.handle_snapshot(
+            vim.vm.ConfigSpec(),
+            obj_ref2,
+            vim.vm.RelocateSpec(),
+            True,
+            {'snapshot': {
+                'disk_move_type': clone_type}})
+
+        self.assertEqual(clone_spec2.location.diskMoveType, clone_type)
+
 if __name__ == '__main__':
     from integration import run_tests
     run_tests(VMwareTestCase, needs_daemon=False)
+    run_tests(CloneFromSnapshotTest, needs_daemon=False)
