@@ -78,8 +78,7 @@ class AlternativesTestCase(TestCase):
             os_readlink_mock.side_effect = OSError('Hell was not found!!!')
             self.assertFalse(alternatives.show_current('hell'))
             os_readlink_mock.assert_called_with('/etc/alternatives/hell')
-            self.assertIn('ERROR:alternatives: path /etc/alternatives/hell '
-                          'does not exist',
+            self.assertIn('ERROR:alternative: hell does not exist',
                           handler.messages)
 
     @patch('os.readlink')
