@@ -900,23 +900,66 @@ class PipTestCase(TestCase):
             }
         )
         with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            ret = pip.freeze()
-            mock.assert_called_once_with(
-                ['pip', 'freeze'],
-                cwd=None,
-                runas=None,
-                use_vt=False,
-                python_shell=False,
-            )
-            self.assertEqual(ret, eggs)
+            with patch('salt.modules.pip.version',
+                       MagicMock(return_value='6.1.1')):
+                ret = pip.freeze()
+                mock.assert_called_once_with(
+                    ['pip', 'freeze'],
+                    cwd=None,
+                    runas=None,
+                    use_vt=False,
+                    python_shell=False,
+                )
+                self.assertEqual(ret, eggs)
 
         # Non zero returncode raises exception?
         mock = MagicMock(return_value={'retcode': 1, 'stderr': 'CABOOOOMMM!'})
         with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            self.assertRaises(
-                CommandExecutionError,
-                pip.freeze,
-            )
+            with patch('salt.modules.pip.version',
+                       MagicMock(return_value='6.1.1')):
+                self.assertRaises(
+                    CommandExecutionError,
+                    pip.freeze,
+                )
+
+    def test_freeze_command_with_all(self):
+        eggs = [
+            'M2Crypto==0.21.1',
+            '-e git+git@github.com:s0undt3ch/salt-testing.git@9ed81aa2f918d59d3706e56b18f0782d1ea43bf8#egg=SaltTesting-dev',
+            'bbfreeze==1.1.0',
+            'bbfreeze-loader==1.1.0',
+            'pip==0.9.1',
+            'pycrypto==2.6',
+            'setuptools==20.10.1'
+        ]
+        mock = MagicMock(
+            return_value={
+                'retcode': 0,
+                'stdout': '\n'.join(eggs)
+            }
+        )
+        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
+            with patch('salt.modules.pip.version',
+                       MagicMock(return_value='9.0.1')):
+                ret = pip.freeze()
+                mock.assert_called_once_with(
+                    ['pip', 'freeze', '--all'],
+                    cwd=None,
+                    runas=None,
+                    use_vt=False,
+                    python_shell=False,
+                )
+                self.assertEqual(ret, eggs)
+
+        # Non zero returncode raises exception?
+        mock = MagicMock(return_value={'retcode': 1, 'stderr': 'CABOOOOMMM!'})
+        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
+            with patch('salt.modules.pip.version',
+                       MagicMock(return_value='9.0.1')):
+                self.assertRaises(
+                    CommandExecutionError,
+                    pip.freeze,
+                )
 
     def test_list_command(self):
         eggs = [
@@ -937,6 +980,7 @@ class PipTestCase(TestCase):
                     cwd=None,
                     runas=None,
                     python_shell=False,
+                    use_vt=False,
                 )
                 self.assertEqual(
                     ret, {
@@ -959,6 +1003,54 @@ class PipTestCase(TestCase):
                     pip.list_,
                 )
 
+    def test_list_command_with_all(self):
+        eggs = [
+            'M2Crypto==0.21.1',
+            '-e git+git@github.com:s0undt3ch/salt-testing.git@9ed81aa2f918d59d3706e56b18f0782d1ea43bf8#egg=SaltTesting-dev',
+            'bbfreeze==1.1.0',
+            'bbfreeze-loader==1.1.0',
+            'pip==9.0.1',
+            'pycrypto==2.6',
+            'setuptools==20.10.1'
+        ]
+        # N.B.: this is deliberately different from the "output" of pip freeze.
+        # This is to demonstrate that the version reported comes from freeze
+        # instead of from the pip.version function.
+        mock_version = '9.0.0'
+        mock = MagicMock(return_value={'retcode': 0, 'stdout': '\n'.join(eggs)})
+        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
+            with patch('salt.modules.pip.version',
+                       MagicMock(return_value=mock_version)):
+                ret = pip.list_()
+                mock.assert_called_with(
+                    ['pip', 'freeze', '--all'],
+                    cwd=None,
+                    runas=None,
+                    python_shell=False,
+                    use_vt=False,
+                )
+                self.assertEqual(
+                    ret, {
+                        'SaltTesting-dev': 'git+git@github.com:s0undt3ch/salt-testing.git@9ed81aa2f918d59d3706e56b18f0782d1ea43bf8',
+                        'M2Crypto': '0.21.1',
+                        'bbfreeze-loader': '1.1.0',
+                        'bbfreeze': '1.1.0',
+                        'pip': '9.0.1',
+                        'pycrypto': '2.6',
+                        'setuptools': '20.10.1'
+                    }
+                )
+
+        # Non zero returncode raises exception?
+        mock = MagicMock(return_value={'retcode': 1, 'stderr': 'CABOOOOMMM!'})
+        with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
+            with patch('salt.modules.pip.version',
+                       MagicMock(return_value='6.1.1')):
+                self.assertRaises(
+                    CommandExecutionError,
+                    pip.list_,
+                )
+
     def test_list_command_with_prefix(self):
         eggs = [
             'M2Crypto==0.21.1',
@@ -974,19 +1066,22 @@ class PipTestCase(TestCase):
             }
         )
         with patch.dict(pip.__salt__, {'cmd.run_all': mock}):
-            ret = pip.list_(prefix='bb')
-            mock.assert_called_with(
-                ['pip', 'freeze'],
-                cwd=None,
-                runas=None,
-                python_shell=False,
-            )
-            self.assertEqual(
-                ret, {
-                    'bbfreeze-loader': '1.1.0',
-                    'bbfreeze': '1.1.0',
-                }
-            )
+            with patch('salt.modules.pip.version',
+                       MagicMock(return_value='6.1.1')):
+                ret = pip.list_(prefix='bb')
+                mock.assert_called_with(
+                    ['pip', 'freeze'],
+                    cwd=None,
+                    runas=None,
+                    python_shell=False,
+                    use_vt=False,
+                )
+                self.assertEqual(
+                    ret, {
+                        'bbfreeze-loader': '1.1.0',
+                        'bbfreeze': '1.1.0',
+                    }
+                )
 
     def test_install_pre_argument_in_resulting_command(self):
         pkg = 'pep8'
