@@ -12,15 +12,17 @@ ensure_in_syspath('../../')
 
 # Import Salt Libs
 from salt.pillar import mysql
+from salt.ext.six import PY3
 
 
 @skipIf(NO_MOCK, NO_MOCK_REASON)
+@skipIf(PY3, 'MySQL-python is not compatible with python3')
 class MysqlPillarTestCase(TestCase):
     maxDiff = None
 
     def test_001_extract_queries_legacy(self):
         return_data = mysql.MySQLExtPillar()
-        args, kwargs = [], {'mysql_query': 'SELECT blah'}
+        args, kwargs = ['SELECT blah'], {}
         qbuffer = return_data.extract_queries(args, kwargs)
         self.assertEqual([
             [None, {'query': 'SELECT blah', 'depth': 0, 'as_list': False,
@@ -100,13 +102,10 @@ class MysqlPillarTestCase(TestCase):
         ], {
             '1': 'SELECT blah1',
             '2': ('SELECT blah2', 2),
-            'mysql_query': 'SELECT blahm',  # In the middle for complexity
             '3': {'query': 'SELECT blah3', 'as_list': True},
         }
         qbuffer = return_data.extract_queries(args, kwargs)
         self.assertEqual([
-            [None, {'query': 'SELECT blahm', 'depth': 0, 'as_list': False,
-                    'with_lists': None, 'ignore_null': False}],
             [None, {'query': 'SELECT blah1', 'depth': 0, 'as_list': False,
                     'with_lists': None, 'ignore_null': False}],
             [None, {'query': 'SELECT blah2', 'depth': 2, 'as_list': False,
