@@ -2,10 +2,19 @@
 
 # Import python libs
 from __future__ import absolute_import
+from datetime import datetime
 import json
 import pprint
 import tempfile
 
+try:
+    from dateutil.relativedelta import relativedelta
+    HAS_DATEUTIL = True
+except ImportError:
+    HAS_DATEUTIL = False
+
+NO_DATEUTIL_REASON = 'python-dateutil is not installed'
+
 # Import Salt Testing libs
 from salttesting import skipIf, TestCase
 from salttesting.helpers import destructiveTest, ensure_in_syspath
@@ -13,6 +22,7 @@ from salttesting.mock import (
     NO_MOCK,
     NO_MOCK_REASON,
     MagicMock,
+    call,
     mock_open,
     patch)
 
@@ -128,7 +138,7 @@ class TestFileState(TestCase):
         # If the test is failing, check the position of the "contents" param
         # in the manage_file() function in salt/modules/file.py, the fix is
         # likely as simple as updating the 2nd index below.
-        self.assertEqual(expected, returner.call_args[0][-4])
+        self.assertEqual(expected, returner.call_args[0][-5])
 
 
 @skipIf(NO_MOCK, NO_MOCK_REASON)
@@ -712,6 +722,12 @@ class FileTestCase(TestCase):
         ret.update({'comment': comt, 'name': ''})
         self.assertDictEqual(filestate.directory(''), ret)
 
+        comt = ('Cannot specify both max_depth and clean')
+        ret.update({'comment': comt, 'name': name})
+        self.assertDictEqual(
+                filestate.directory(name, clean=True, max_depth=2),
+                ret)
+
         mock_t = MagicMock(return_value=True)
         mock_f = MagicMock(return_value=False)
         mock_perms = MagicMock(return_value=(ret, ''))
@@ -874,7 +890,7 @@ class FileTestCase(TestCase):
                 ret.update({'comment': comt})
                 self.assertDictEqual(filestate.recurse(name, source), ret)
 
-                comt = ("The directory 'salt://code/flask' does not exist"
+                comt = ("The directory 'code/flask' does not exist"
                         " on the salt fileserver in saltenv 'base'")
                 ret.update({'comment': comt})
                 self.assertDictEqual(filestate.recurse(name, source), ret)
@@ -989,7 +1005,6 @@ class FileTestCase(TestCase):
 
         mock_t = MagicMock(return_value=True)
         mock_f = MagicMock(return_value=False)
-        mock = MagicMock(side_effect=[False, True, False, False])
         with patch.object(os.path, 'isabs', mock_f):
             comt = ('Specified file {0} is not an absolute path'.format(name))
             ret.update({'comment': comt, 'name': name})
@@ -997,8 +1012,7 @@ class FileTestCase(TestCase):
 
         with patch.object(os.path, 'isabs', mock_t):
             with patch.dict(filestate.__salt__,
-                            {'file.contains_regex_multiline': mock,
-                             'file.search': mock}):
+                            {'file.search': MagicMock(side_effect=[True, True, True, False, False])}):
                 comt = ('Pattern already commented')
                 ret.update({'comment': comt, 'result': True})
                 self.assertDictEqual(filestate.comment(name, regex), ret)
@@ -1008,8 +1022,7 @@ class FileTestCase(TestCase):
                 self.assertDictEqual(filestate.comment(name, regex), ret)
 
             with patch.dict(filestate.__salt__,
-                            {'file.contains_regex_multiline': mock_t,
-                             'file.search': mock_t,
+                            {'file.search': MagicMock(side_effect=[False, True, False, True, True]),
                              'file.comment': mock_t,
                              'file.comment_line': mock_t}):
                 with patch.dict(filestate.__opts__, {'test': True}):
@@ -1057,8 +1070,7 @@ class FileTestCase(TestCase):
 
         with patch.object(os.path, 'isabs', mock_t):
             with patch.dict(filestate.__salt__,
-                            {'file.contains_regex_multiline': mock,
-                             'file.search': mock,
+                            {'file.search': mock,
                              'file.uncomment': mock_t,
                              'file.comment_line': mock_t}):
                 comt = ('Pattern already uncommented')
@@ -1114,7 +1126,6 @@ class FileTestCase(TestCase):
                          'file.makedirs': mock_t,
                          'file.stats': mock_f,
                          'cp.get_template': mock_f,
-                         'file.contains_regex_multiline': mock_f,
                          'file.search': mock_f,
                          'file.prepend': mock_t}):
             with patch.object(os.path, 'isdir', mock_t):
@@ -1651,6 +1662,154 @@ class FileTestCase(TestCase):
 
             self.assertTrue(filestate.mod_run_check_cmd(cmd, filename))
 
+    @skipIf(not HAS_DATEUTIL, NO_DATEUTIL_REASON)
+    def test_retention_schedule(self):
+        '''
+        Test to execute the retention_schedule logic.
+
+        This test takes advantage of knowing which files it is generating,
+        which means it can easily generate list of which files it should keep.
+        '''
+
+        def generate_fake_files(format='example_name_%Y%m%dT%H%M%S.tar.bz2',
+                                starting=datetime(2016, 2, 8, 9),
+                                every=relativedelta(minutes=30),
+                                ending=datetime(2015, 12, 25),
+                                maxfiles=None):
+            '''
+            For starting, make sure that it's over a week from the beginning of the month
+            For every, pick only one of minutes, hours, days, weeks, months or years
+            For ending, the further away it is from starting, the slower the tests run
+            Full coverage requires over a year of separation, but that's painfully slow.
+            '''
+
+            if every.years:
+                ts = datetime(starting.year, 1, 1)
+            elif every.months:
+                ts = datetime(starting.year, starting.month, 1)
+            elif every.days:
+                ts = datetime(starting.year, starting.month, starting.day)
+            elif every.hours:
+                ts = datetime(starting.year, starting.month, starting.day, starting.hour)
+            elif every.minutes:
+                ts = datetime(starting.year, starting.month, starting.day, starting.hour, 0)
+            else:
+                raise NotImplementedError("not sure what you're trying to do here")
+
+            fake_files = []
+            count = 0
+            while ending < ts:
+                fake_files.append(ts.strftime(format=format))
+                count += 1
+                if maxfiles and count >= maxfiles:
+                    break
+                ts -= every
+            return fake_files
+
+        fake_name = '/some/dir/name'
+        fake_retain = {
+            'most_recent': 2,
+            'first_of_hour': 4,
+            'first_of_day': 7,
+            'first_of_week': 6,
+            'first_of_month': 6,
+            'first_of_year': 'all',
+        }
+        fake_strptime_format = 'example_name_%Y%m%dT%H%M%S.tar.bz2'
+        fake_matching_file_list = generate_fake_files()
+        # Add some files which do not match fake_strptime_format
+        fake_no_match_file_list = generate_fake_files(format='no_match_%Y%m%dT%H%M%S.tar.bz2',
+                                                      every=relativedelta(days=1))
+
+        def lstat_side_effect(path):
+            import re
+            from time import mktime
+            x = re.match(r'^[^\d]*(\d{8}T\d{6})\.tar\.bz2$', path).group(1)
+            ts = mktime(datetime.strptime(x, '%Y%m%dT%H%M%S').timetuple())
+            return {'st_atime': 0.0, 'st_ctime': 0.0, 'st_gid': 0,
+                    'st_mode': 33188, 'st_mtime': ts,
+                    'st_nlink': 1, 'st_size': 0, 'st_uid': 0,
+                   }
+
+        mock_t = MagicMock(return_value=True)
+        mock_f = MagicMock(return_value=False)
+        mock_lstat = MagicMock(side_effect=lstat_side_effect)
+        mock_remove = MagicMock()
+
+        def run_checks(isdir=mock_t, strptime_format=None, test=False):
+            expected_ret = {
+                    'name': fake_name,
+                    'changes': {'retained': [], 'deleted': [], 'ignored': []},
+                    'pchanges': {'retained': [], 'deleted': [], 'ignored': []},
+                    'result': True,
+                    'comment': 'Name provided to file.retention must be a directory',
+                }
+            if strptime_format:
+                fake_file_list = sorted(fake_matching_file_list + fake_no_match_file_list)
+            else:
+                fake_file_list = sorted(fake_matching_file_list)
+            mock_readdir = MagicMock(return_value=fake_file_list)
+
+            with patch.dict(filestate.__opts__, {'test': test}):
+                with patch.object(os.path, 'isdir', isdir):
+                    mock_readdir.reset_mock()
+                    with patch.dict(filestate.__salt__, {'file.readdir': mock_readdir}):
+                        with patch.dict(filestate.__salt__, {'file.lstat': mock_lstat}):
+                            mock_remove.reset_mock()
+                            with patch.dict(filestate.__salt__, {'file.remove': mock_remove}):
+                                if strptime_format:
+                                    actual_ret = filestate.retention_schedule(fake_name, fake_retain,
+                                                                              strptime_format=fake_strptime_format)
+                                else:
+                                    actual_ret = filestate.retention_schedule(fake_name, fake_retain)
+
+            if not isdir():
+                mock_readdir.assert_has_calls([])
+                expected_ret['result'] = False
+            else:
+                mock_readdir.assert_called_once_with(fake_name)
+                ignored_files = fake_no_match_file_list if strptime_format else []
+                retained_files = set(generate_fake_files(maxfiles=fake_retain['most_recent']))
+                junk_list = [('first_of_hour', relativedelta(hours=1)),
+                             ('first_of_day', relativedelta(days=1)),
+                             ('first_of_week', relativedelta(weeks=1)),
+                             ('first_of_month', relativedelta(months=1)),
+                             ('first_of_year', relativedelta(years=1))]
+                for retainable, retain_interval in junk_list:
+                    new_retains = set(generate_fake_files(maxfiles=fake_retain[retainable], every=retain_interval))
+                    # if we generate less than the number of files expected,
+                    # then the oldest file will also be retained
+                    # (correctly, since it's the first in it's category)
+                    if len(new_retains) < fake_retain[retainable]:
+                        new_retains.add(fake_file_list[0])
+                    retained_files |= new_retains
+
+                deleted_files = sorted(list(set(fake_file_list) - retained_files - set(ignored_files)), reverse=True)
+                retained_files = sorted(list(retained_files), reverse=True)
+                changes = {'retained': retained_files, 'deleted': deleted_files, 'ignored': ignored_files}
+                expected_ret['pchanges'] = changes
+                if test:
+                    expected_ret['result'] = None
+                    expected_ret['comment'] = ('{0} backups would have been removed from {1}.\n'
+                                               ''.format(len(deleted_files), fake_name))
+                else:
+                    expected_ret['comment'] = ('{0} backups were removed from {1}.\n'
+                                               ''.format(len(deleted_files), fake_name))
+                    expected_ret['changes'] = changes
+                    mock_remove.assert_has_calls(
+                            [call(os.path.join(fake_name, x)) for x in deleted_files],
+                            any_order=True
+                        )
+
+            self.assertDictEqual(actual_ret, expected_ret)
+
+        run_checks(isdir=mock_f)
+        run_checks()
+        run_checks(test=True)
+        run_checks(strptime_format=fake_strptime_format)
+        run_checks(strptime_format=fake_strptime_format, test=True)
+
+
 if __name__ == '__main__':
     from integration import run_tests
     run_tests(FileTestCase, needs_daemon=False)
