@@ -768,6 +768,10 @@ class UtilsTestCase(TestCase):
             ut = bytes((0xe4, 0xb8, 0xad, 0xe5, 0x9b, 0xbd, 0xe8, 0xaa, 0x9e, 0x20, 0x28, 0xe7, 0xb9, 0x81, 0xe4, 0xbd, 0x93, 0x29))
             self.assertEqual(utils.to_str(ut, 'utf-8'), un)
             self.assertEqual(utils.to_str(bytearray(ut), 'utf-8'), un)
+            # Test situation when a minion returns incorrect utf-8 string because of... million reasons
+            ut2 = b'\x9c'
+            self.assertEqual(utils.to_str(ut2, 'utf-8'), u'\ufffd')
+            self.assertEqual(utils.to_str(bytearray(ut2), 'utf-8'), u'\ufffd')
         else:
             self.assertEqual(utils.to_str('plugh'), 'plugh')
             self.assertEqual(utils.to_str(u'áéíóúý', 'utf-8'), 'áéíóúý')
