@@ -99,7 +99,6 @@ class TestVerify(TestCase):
         self.assertTrue(os.path.exists(var_dir))
         dir_stat = os.stat(var_dir)
         self.assertEqual(dir_stat.st_uid, os.getuid())
-        self.assertEqual(dir_stat.st_gid, os.getgid())
         self.assertEqual(dir_stat.st_mode & stat.S_IRWXU, stat.S_IRWXU)
         self.assertEqual(dir_stat.st_mode & stat.S_IRWXG, 40)
         self.assertEqual(dir_stat.st_mode & stat.S_IRWXO, 5)
@@ -231,17 +230,17 @@ class TestVerify(TestCase):
         message = 'Insecure logging configuration detected! Sensitive data may be logged.'
 
         mock_cheese = MagicMock()
-        with patch.object(log, 'warn', mock_cheese):
+        with patch.object(log, 'warning', mock_cheese):
             verify_log({'log_level': 'cheeseshop'})
             mock_cheese.assert_called_once_with(message)
 
         mock_trace = MagicMock()
-        with patch.object(log, 'warn', mock_trace):
+        with patch.object(log, 'warning', mock_trace):
             verify_log({'log_level': 'trace'})
             mock_trace.assert_called_once_with(message)
 
         mock_info = MagicMock()
-        with patch.object(log, 'warn', mock_info):
+        with patch.object(log, 'warning', mock_info):
             verify_log({'log_level': 'info'})
             mock_info.assert_not_called()
 
